<?php
/**
  * Creates proper tables for M2F
  *
  * @author Patricio Anguita
  * @license http://opensource.org/licenses/gpl-license.php GNU Public License
  * @version $Revision: 1.16 $
  * @internal $Id: m2f_schema_mysql.php,v 1.16 2006/04/28 18:39:15 georgeocrawford Exp $
*/

//
//
// Whenever the schema is changed in any way, be sure to increment the $m2f_desired_db_version
// value in m2f_common.php!!!!!
//
//



//
// Create the DB tables
//
function m2f_create_tables_mysql($m2f_desired_db_version, $m2f_distributionlist_settings, $m2f_config_settings)
{
	global $m2f_db;
	
	// Create M2F tables
	$sql = "CREATE TABLE " . _M2F_CONFIG_TABLE_ . "
					(
	  				config_name varchar(64) default '',
	  				config_value text default ''
	   			)";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);
	
	foreach ($m2f_config_settings as $config_name => $config_value)
	{
		$sql = "INSERT INTO " . _M2F_CONFIG_TABLE_ . "
							(config_name, config_value)
						VALUES ('$config_name', '$config_value')";
		$db_result = $m2f_db->query($sql);
		check_db_result($db_result, $sql, __LINE__, __FILE__);
	}
	
	$sql = "CREATE TABLE " . _M2F_DISTRIBUTIONLIST_CONFIG_TABLE_ . "
					(
	  				distributionlist_id INTEGER NOT NULL default '0',
	  				config_name varchar(64) NOT NULL default '',
	  				config_value varchar(254) NOT NULL default '',
	  				PRIMARY KEY (distributionlist_id, config_name, config_value)
	  			)";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);
	
	$sql = "CREATE TABLE " . _M2F_CRON_TABLE_ . "
				(
					`cron_id` int(11) NOT NULL auto_increment,
					`logtime` int(10) NOT NULL,
					`results` text NOT NULL,
					`status` tinyint(4) NOT NULL default '0',
					PRIMARY KEY  (cron_id)
				)";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);
	
	$sql = "CREATE TABLE " . _M2F_MESSAGE_HASH_TABLE_ . "
				(
					`hash` varchar(255) NOT NULL default '',
					`logtime` int(10) NOT NULL default '0',
					`id` int(10) unsigned,
					`tag` char(3) default NULL,
					UNIQUE KEY `hash` (`hash`)
				)";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);
	
	$sql = "CREATE TABLE " . _M2F_MESSAGE_ID_TABLE_ . "
					(
						`post_id` mediumint(8) NOT NULL default '0',
						`message_id` varchar(255) NOT NULL default '',
						`logtime` int(10) NOT NULL default '0'
	  			)";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);

	$sql = "CREATE TABLE " . _M2F_DISTRIBUTIONLIST_TABLE_ . "
					(
	  				distributionlist_id INTEGER NOT NULL default '0',
	  				mailinglist_id INTEGER default '0',
	  				forum_id INTEGER default '0',
	  				status SMALLINT default '0',
	  				visibility SMALLINT default '0',
	  				PRIMARY KEY (distributionlist_id)
	  			)";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);
	
	$sql = "CREATE TABLE " . _M2F_SUBSCRIPTION_TABLE_ . "
					(
						mailinglist_id INTEGER NOT NULL default '0',
						user_id INTEGER NOT NULL default '0',
						PRIMARY KEY (mailinglist_id,user_id)
					)";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);
	
	$sql = "CREATE TABLE " . _M2F_GROUP_SUBSCRIPTION_TABLE_ . "
					(
						mailinglist_id INTEGER NOT NULL default '0',
						group_id INTEGER NOT NULL default '0',
						PRIMARY KEY (mailinglist_id, group_id)
					)";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);

	$sql = "CREATE TABLE " . _M2F_MAILINGLIST_TABLE_ . "
					(
						mailinglist_id INTEGER NOT NULL default '0',
						name VARCHAR( 254 ) default '',
						list_email VARCHAR( 254 ) default '',
						subscribe_board SMALLINT default '0',
						PRIMARY KEY ( mailinglist_id )
					)";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);

	$sql = "CREATE TABLE " . _M2F_LOG_TABLE_ . "
				(
					id int(11) NOT NULL default '0', 
					logtime timestamp NOT NULL, 
					ident varchar(16) default '', 
					priority int(11) default '0', 
					message text, 
					PRIMARY KEY (id) 
				)"; 
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);


	$sql = "CREATE TABLE " . _M2F_MODERATED_MSGS_TABLE_ . "
				(
					`id` int(10) NOT NULL default '0',
					`distributionlist_id` smallint(5) NOT NULL default '0',
					`mailinglist_id` smallint(5) NOT NULL default '0',
					`forum_id` smallint(5) NOT NULL default '0',
					`topic_id` smallint(8) NOT NULL default '0',
					`post_id` smallint(8) NOT NULL default '0',
					`user_id` smallint(8) NOT NULL default '0',
					`forum_msg` text,
					`mail_msg` longtext,
					`userdata` text,
					`recipient_list` longtext,
					`direction` char(3),
					`time` int(10) NOT NULL default '0',
					`reason` varchar(15) default NULL,
					PRIMARY KEY (id) 
				)"; 
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);


	// Set up default distributionlist configuration table
	foreach ($m2f_distributionlist_settings as $k => $v)
	{
		if (is_array($v))
		{
			foreach ($v as $b)
			{
				$sql = "INSERT INTO " . _M2F_DISTRIBUTIONLIST_CONFIG_TABLE_ . "
									(distributionlist_id, config_name, config_value)
								VALUES ('0', '$k', '$b')";
				$db_result = $m2f_db->query($sql);
				check_db_result($db_result, $sql, __LINE__, __FILE__);
			}
		}
		else
		{
			$sql = "INSERT INTO " . _M2F_DISTRIBUTIONLIST_CONFIG_TABLE_ . "
								(distributionlist_id, config_name, config_value)
							VALUES ('0', '$k', '$v')" ;
			$db_result = $m2f_db->query($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);
		}      
	}
	
	$sql = "CREATE TABLE " . _M2F_PATTERN_MATCH_TABLE_ . " 
				(
					`pattern_id` int(11) NOT NULL auto_increment,
					`pattern_match` text,
					`pattern_replace` text,
					`use_regexp` tinyint(1) NOT NULL default '0',
					`modifiers` varchar(10) default NULL,
					`description` text,
					`target` varchar(20) default NULL,
					`internal` tinyint(1) NOT NULL default '0',
					`purpose` varchar(6) NOT NULL default 'search',
					PRIMARY KEY  (`pattern_id`)
				)";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);

	$sql = "CREATE TABLE " . _M2F_DIST_LIST_PATTERNS_TABLE_ . "
				(
					`distributionlist_id` int(11) NOT NULL default '0',
					`pattern_id` int(11) NOT NULL default '0',
					`pattern_order` int(11) NOT NULL default '0',
					PRIMARY KEY  (`distributionlist_id`,`pattern_id`)
				)";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);

	// insert hardcoded Yahoo Groups/Spam detection patterns
	$sql = 'INSERT INTO ' . _M2F_PATTERN_MATCH_TABLE_ . '  
					(`pattern_match`,`pattern_replace`,`use_regexp`,`modifiers`,`description`,`target`,`internal`, `purpose`) 
					VALUES 
					("' . $m2f_db->escapeSimple('\s*<!-- [^>]+begin egp html banner.+end egp html banner[^>]+ -->\s*') . '","\n","1","s","*Yahoo Groups* Remove HTML-style Yahoo Groups footer in HTML body","2","1", "search"), 
					("' . $m2f_db->escapeSimple('\s*Yahoo! Groups Links.+http:\/\/docs\.yahoo\.com\/info\/terms\/\s*') . '","\n","1","s","*Yahoo Groups* Remove text-style Yahoo Groups footers in text body","2","1", "search"),
					("' . $m2f_db->escapeSimple('(?<=\W)[>\n\r -]+\*?Yahoo! Groups Sponsor.+(?=(?i)<\/body>)') . '","\n","1","sm","*Yahoo Groups* Remove text-style Yahoo Groups footer in HTML body","2","1", "search"),
					("' . $m2f_db->escapeSimple('(?<=\W)[>\n\r -]+\*?Yahoo! Groups Sponsor.+') . '","\n","1","sm","*Yahoo Groups* Remove text-style Yahoo Groups footer in text body","2","1", "search"),
					("Your email requires verification","","0","","Block emails with \"Your email requires verification\" in Subject","1","1","block"),
					("Mail delivery failed: returning message to sender","","0","","Block emails with \"Mail delivery failed: returning message to sender\" in Subject","1","1","block"),
					("Autoreply","","0","","Block emails with \"Autoreply\" in Subject","1","1","block"),
					("Undelivered Mail Returned to Sender","","0","","Block emails with \"Undelivered Mail Returned to Sender\" in Subject","1","1","block"),
					("failure notice","","0","","Block emails with \"failure notice\" in Subject","1","1","block")';
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);





	//
	//  Alter phpBB table
	//
	
	$sql = "ALTER TABLE " . USERS_TABLE . "
					ADD COLUMN m2f_user_prefs int(1) NOT NULL DEFAULT '-1'";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);

}



//
// Adds a new distribution list
//
function m2f_create_distributionlist()
{
	  global $m2f_db;
	  
		// Populate m2f_distributionlist, and get the new id
		$sql = "SELECT MAX(distributionlist_id) FROM " . _M2F_DISTRIBUTIONLIST_TABLE_;
		$db_result = $m2f_db->getOne($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
    
		$distributionlist_id = $db_result + 1;
		
		// Insert a new row
		$sql = "INSERT INTO " . _M2F_DISTRIBUTIONLIST_TABLE_ . " (distributionlist_id)
						VALUES ('$distributionlist_id')";
		$db_result = $m2f_db->query($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);

		
		// Get the settings from the default distribution list (id = 0)
		$sql = "SELECT * FROM " . _M2F_DISTRIBUTIONLIST_CONFIG_TABLE_ . " WHERE distributionlist_id = 0";
		$db_result = $m2f_db->getAll($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
		
		// Set up distributionlist configuration table
    foreach ($db_result as $config)
    {
			$sql = "INSERT INTO " . _M2F_DISTRIBUTIONLIST_CONFIG_TABLE_ . "
								(distributionlist_id, config_name, config_value)
							VALUES ('$distributionlist_id', '$config[config_name]', '$config[config_value]')" ;
			$db_result = $m2f_db->query($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);
    }

		// Get the linked patterns from the default distribution list (id = 0)
		$sql = "SELECT * FROM " . _M2F_DIST_LIST_PATTERNS_TABLE_ . " WHERE distributionlist_id = 0";
		$db_result = $m2f_db->getAll($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
		
		// Set up pattern links for new list
    foreach ($db_result as $pattern)
    {
			$sql = "INSERT INTO " . _M2F_DIST_LIST_PATTERNS_TABLE_ . "
								(`distributionlist_id`, `pattern_id`, `pattern_order`)
							VALUES ('$distributionlist_id', '$pattern[pattern_id]', '$pattern[pattern_order]')" ;
			$db_result = $m2f_db->query($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);
    }
    
		// Return the new id
		return $distributionlist_id;
}

?>