<?php
/**
	* Creates proper tables for M2F
	*
	* @author Patricio Anguita
	* @license http://opensource.org/licenses/gpl-license.php GNU Public License
	* @version $Revision: 1.15 $
	* @internal $Id: m2f_schema_pgsql.php,v 1.15 2006/04/28 18:39:15 georgeocrawford Exp $
*/

//
//
// Whenever the schema is changed in any way, be sure to increment the $m2f_desired_db_version
// value in m2f_common.php!!!!!
//
//

//
// Create the DB tables
//
function m2f_create_tables_pgsql($m2f_desired_db_version, $m2f_distributionlist_settings, $m2f_config_settings)
{
	global $m2f_db;

	// Create M2F tables
	$sql = "CREATE TABLE " . _M2F_CONFIG_TABLE_ . "
					(
						config_name VARCHAR(64) DEFAULT '',
						config_value TEXT DEFAULT '',
						PRIMARY KEY (config_name)
					)";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);
	
	foreach ($m2f_config_settings as $config_name => $config_value)
	{
		$sql = "INSERT INTO " . _M2F_CONFIG_TABLE_ . "
							(config_name, config_value)
						VALUES ('$config_name', '$config_value')";
		$db_result = $m2f_db->query($sql);
		check_db_result($db_result, $sql, __LINE__, __FILE__);
	}
	
	$sql = "CREATE TABLE " . _M2F_DISTRIBUTIONLIST_CONFIG_TABLE_ . "
					(
						distributionlist_id INTEGER NOT NULL DEFAULT 0,
						config_name VARCHAR(64) NOT NULL DEFAULT '',
						config_value VARCHAR(254) NOT NULL DEFAULT '',
						PRIMARY KEY (distributionlist_id, config_name, config_value)
					)";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);
	
	$sql = "CREATE TABLE " . _M2F_CRON_TABLE_ . "
				(
					cron_id SERIAL NOT NULL,
					logtime INTEGER NOT NULL,
					results TEXT NOT NULL,
					status SMALLINT NOT NULL DEFAULT 0,
					PRIMARY KEY	(cron_id)
				)";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);
	
	$sql = "CREATE TABLE " . _M2F_MESSAGE_HASH_TABLE_ . "
				(
					hash VARCHAR(255) NOT NULL DEFAULT '' UNIQUE,
					logtime INTEGER NOT NULL,
					id SERIAL NOT NULL,
					post_id INTEGER,
					tag VARCHAR(3) default NULL,
				)";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);
	
	$sql = "CREATE TABLE " . _M2F_MESSAGE_ID_TABLE_ . "
					(
						post_id INTEGER NOT NULL,
						message_id VARCHAR(255) NOT NULL DEFAULT '',
						logtime INTEGER NOT NULL
	  			)";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);

	$sql = "CREATE TABLE " . _M2F_DISTRIBUTIONLIST_TABLE_ . "
					(
						distributionlist_id INTEGER NOT NULL DEFAULT 0,
						mailinglist_id INTEGER DEFAULT 0,
						forum_id INTEGER DEFAULT 0,
						status SMALLINT DEFAULT 0,
						visibility SMALLINT DEFAULT 0,
						PRIMARY KEY (distributionlist_id)
					)";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);
	
	$sql = "CREATE TABLE " . _M2F_SUBSCRIPTION_TABLE_ . "
					(
						mailinglist_id INTEGER NOT NULL DEFAULT 0,
						user_id INTEGER NOT NULL DEFAULT 0,
						PRIMARY KEY (mailinglist_id,user_id)
					)";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);
	
	$sql = "CREATE TABLE " . _M2F_GROUP_SUBSCRIPTION_TABLE_ . "
					(
						mailinglist_id INTEGER NOT NULL DEFAULT 0,
						group_id INTEGER NOT NULL DEFAULT 0,
						PRIMARY KEY (mailinglist_id, group_id)
					)";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);

	$sql = "CREATE TABLE " . _M2F_MAILINGLIST_TABLE_ . "
					(
						mailinglist_id INTEGER NOT NULL DEFAULT 0,
						name VARCHAR( 254 ) DEFAULT '',
						list_email VARCHAR( 254 ) DEFAULT '',
						subscribe_board SMALLINT DEFAULT 0,
						PRIMARY KEY ( mailinglist_id )
					)";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);

	$sql = "CREATE TABLE " . _M2F_LOG_TABLE_ . "
				(
					id INTEGER NOT NULL DEFAULT 0, 
					logtime TIMESTAMP WITH TIME ZONE DEFAULT CURRENT_TIMESTAMP, 
					ident VARCHAR(16) DEFAULT '', 
					priority INTEGER DEFAULT 0, 
					message TEXT, 
					PRIMARY KEY (id) 
				)"; 
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);

	$sql = "CREATE TABLE " . _M2F_MODERATED_MSGS_TABLE_ . "
				(
					id INTEGER NOT NULL DEFAULT 0, 
					distributionlist_id INTEGER NOT NULL default 0,
					mailinglist_id INTEGER NOT NULL default 0,
					forum_id INTEGER NOT NULL default 0,
					topic_id INTEGER NOT NULL default 0,
					post_id INTEGER NOT NULL default 0,
					user_id INTEGER NOT NULL default 0,
					forum_msg TEXT,
					mail_msg TEXT,
					userdata TEXT,
					recipient_list TEXT,
					direction character(3),
					time INTEGER NOT NULL default 0,
					reason varchar(15) default NULL,
					PRIMARY KEY (id) 
				)"; 
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);

	// Set up default distributionlist configuration table
	foreach ($m2f_distributionlist_settings as $k => $v)
	{
		if (is_array($v))
		{
			foreach ($v as $b)
			{
				$sql = "INSERT INTO " . _M2F_DISTRIBUTIONLIST_CONFIG_TABLE_ . "
									(distributionlist_id, config_name, config_value)
								VALUES (0, '$k', '$b')";
				$db_result = $m2f_db->query($sql);
				check_db_result($db_result, $sql, __LINE__, __FILE__);
			}
		}
		else
		{
			$sql = "INSERT INTO " . _M2F_DISTRIBUTIONLIST_CONFIG_TABLE_ . "
								(distributionlist_id, config_name, config_value)
							VALUES (0, '$k', '$v')" ;
			$db_result = $m2f_db->query($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);
		}			
	}
	
	$sql = "CREATE TABLE " . _M2F_PATTERN_MATCH_TABLE_ . " 
				(
					pattern_id SERIAL NOT NULL,
					pattern_match TEXT,
					pattern_replace TEXT,
					use_regexp SMALLINT NOT NULL DEFAULT 0,
					modifiers VARCHAR(10) DEFAULT NULL,
					description TEXT,
					target VARCHAR(20) DEFAULT NULL,
					internal SMALLINT NOT NULL DEFAULT 0,
					purpose VARCHAR(6) NOT NULL DEFAULT 'search',
					PRIMARY KEY	(pattern_id)
				)";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);

	$sql = "CREATE TABLE " . _M2F_DIST_LIST_PATTERNS_TABLE_ . "
				(
					distributionlist_id INTEGER NOT NULL DEFAULT 0,
					pattern_id INTEGER NOT NULL DEFAULT '0',
					pattern_order INTEGER NOT NULL DEFAULT '0',
					PRIMARY KEY	(distributionlist_id,pattern_id)
				)";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);

	// insert hardcoded Yahoo Groups/Spam detection patterns
	$sql = "INSERT INTO " . _M2F_PATTERN_MATCH_TABLE_ . " (pattern_match,pattern_replace,use_regexp,modifiers,description,target,internal,purpose) 
					VALUES ('" . $m2f_db->escapeSimple('\s*<!-- [^>]+begin egp html banner.+end egp html banner[^>]+ -->\s*') . "','\n',1,'s','*Yahoo Groups* Remove HTML-style Yahoo Groups footer in HTML body','Body',1,'search'); 
					INSERT INTO " . _M2F_PATTERN_MATCH_TABLE_ . " (pattern_match,pattern_replace,use_regexp,modifiers,description,target,internal,purpose)
					VALUES ('" . $m2f_db->escapeSimple('\s*Yahoo! Groups Links.+http:\/\/docs\.yahoo\.com\/info\/terms\/\s*') . "','\n',1,'s','*Yahoo Groups* Remove text-style Yahoo Groups footers in text body','Body',1,'search');
					INSERT INTO " . _M2F_PATTERN_MATCH_TABLE_ . " (pattern_match,pattern_replace,use_regexp,modifiers,description,target,internal,purpose)
					VALUES ('" . $m2f_db->escapeSimple('(?<=\W)[>\n\r -]+\*?Yahoo! Groups Sponsor.+(?=(?i)<\/body>)') . "','\n',1,'sm','*Yahoo Groups* Remove text-style Yahoo Groups footer in HTML body','Body',1,'search');
					INSERT INTO " . _M2F_PATTERN_MATCH_TABLE_ . " (pattern_match,pattern_replace,use_regexp,modifiers,description,target,internal,purpose)
					VALUES ('" . $m2f_db->escapeSimple('(?<=\W)[>\n\r -]+\*?Yahoo! Groups Sponsor.+') . "','\n',1,'sm','*Yahoo Groups* Remove text-style Yahoo Groups footer in text body','Body',1,'search');
					INSERT INTO " . _M2F_PATTERN_MATCH_TABLE_ . " (pattern_match,pattern_replace,use_regexp,modifiers,description,target,internal,purpose)
					VALUES ('Your email requires verification','',0,'','Block emails with \"Your email requires verification\" in Subject','Subject',1,'block');
					INSERT INTO " . _M2F_PATTERN_MATCH_TABLE_ . " (pattern_match,pattern_replace,use_regexp,modifiers,description,target,internal,purpose)
					VALUES ('Autoreply','',0,'','Block emails with \"Autoreply\" in Subject','Subject',1,'block');
					INSERT INTO " . _M2F_PATTERN_MATCH_TABLE_ . " (pattern_match,pattern_replace,use_regexp,modifiers,description,target,internal,purpose)
					VALUES ('Undelivered Mail Returned to Sender','',0,'','Block emails with \"Undelivered Mail Returned to Sender\" in Subject','Subject',1,'block');
					INSERT INTO " . _M2F_PATTERN_MATCH_TABLE_ . " (pattern_match,pattern_replace,use_regexp,modifiers,description,target,internal,purpose)
					VALUES ('failure notice','',0,'','Block emails with \"failure notice\" in Subject','Subject',1,'block');
					INSERT INTO " . _M2F_PATTERN_MATCH_TABLE_ . " (pattern_match,pattern_replace,use_regexp,modifiers,description,target,internal,purpose)
					VALUES ('Mail delivery failed: returning message to sender','',0,'','Block emails with \"Mail delivery failed: returning message to sender\" in Subject','Subject',1,'block')";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);



	//
	//  Alter phpBB table
	//
	
	$sql = "ALTER TABLE " . USERS_TABLE . "
					ADD COLUMN m2f_user_prefs INTEGER NOT NULL DEFAULT '-1'";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);
}



//
// Adds a new distribution list
//
function m2f_create_distributionlist()
{
		global $m2f_db;
		
		// Populate m2f_distributionlist, and get the new id
		$sql = "SELECT MAX(distributionlist_id) FROM " . _M2F_DISTRIBUTIONLIST_TABLE_;
		$db_result = $m2f_db->getOne($sql);
		check_db_result($db_result, $sql, __LINE__, __FILE__);
		
		$distributionlist_id = $db_result + 1;
		
		// Insert a new row
		$sql = "INSERT INTO " . _M2F_DISTRIBUTIONLIST_TABLE_ . " (distributionlist_id)
						VALUES ($distributionlist_id)";
		$db_result = $m2f_db->query($sql);
		check_db_result($db_result, $sql, __LINE__, __FILE__);

		
		// Get the settings from the default distribution list (id = 0)
		$sql = "SELECT * FROM " . _M2F_DISTRIBUTIONLIST_CONFIG_TABLE_ . " WHERE distributionlist_id = 0";
		$db_result = $m2f_db->getAll($sql);
		check_db_result($db_result, $sql, __LINE__, __FILE__);
		
		// Set up distributionlist configuration table
		foreach ($db_result as $config)
		{
			$sql = "INSERT INTO " . _M2F_DISTRIBUTIONLIST_CONFIG_TABLE_ . "
								(distributionlist_id, config_name, config_value)
							VALUES ($distributionlist_id, '$config[config_name]', '$config[config_value]')" ;
			$db_result = $m2f_db->query($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);
		}

		// Get the linked patterns from the default distribution list (id = 0)
		$sql = "SELECT * FROM " . _M2F_DIST_LIST_PATTERNS_TABLE_ . " WHERE distributionlist_id = 0";
		$db_result = $m2f_db->getAll($sql);
		check_db_result($db_result, $sql, __LINE__, __FILE__);
		
		// Set up pattern links for new list
		foreach ($db_result as $pattern)
		{
			$sql = "INSERT INTO " . _M2F_DIST_LIST_PATTERNS_TABLE_ . "
								(distributionlist_id, pattern_id, pattern_order)
							VALUES ($distributionlist_id, '$pattern[pattern_id]', '$pattern[pattern_order]')" ;
			$db_result = $m2f_db->query($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);
		}
		
		// Return the new id
		return $distributionlist_id;
}

?>