<?php
/**
  * Update m2f database schema to version 9
  *
  * @author Patricio Anguita
  * @license http://opensource.org/licenses/gpl-license.php GNU Public License
  * @version
  * @internal
*/



function m2f_update_schema_9($type_db)
{
	global $m2f_db;
 	m2f_log(PEAR_LOG_NOTICE , "Updating M2F tables to DB version 9" , __LINE__ , __FILE__);

	//
	// Adds any missing distribution list settings to the table
	//
	$new_dist_list_fields = array(
		'_M2F_FORCE_GUEST_POSTING_' => '0',
		'_M2F_IGNORE_SECURITY_SETTINGS_' => '0',
		'_M2F_MAX_MESSAGE_SIZE_' => '0',
		'_M2F_MODERATED_LIST_IN_' => '0',
		'_M2F_MODERATED_LIST_OUT_' => '0',
		'_M2F_MODERATOR_IGNORE_ADMIN_' => '0',
		'_M2F_MODERATOR_NOTIFY_' => 'none',
		'_M2F_MODERATOR_NOTIFY_EMAIL_' => '',
		'_M2F_ADD_AUTHOR_AS_RECIPIENT_' => '1',
	);
	
	// Get list of current distribution lists
	$sql = "SELECT distributionlist_id FROM " . _M2F_DISTRIBUTIONLIST_TABLE_;
	$lists = array_merge($m2f_db->getCol($sql), array('0')); // make sure we do the default dist list too!
	check_db_result($lists, $sql, __LINE__, __FILE__);
	
	foreach ($lists as $id)
	{
		$sql = "SELECT config_name, config_value
						FROM " . _M2F_DISTRIBUTIONLIST_CONFIG_TABLE_ . "
						WHERE distributionlist_id = $id";
		$existing_configs =& $m2f_db->getAssoc($sql, false, array(), DB_FETCHMODE_DEFAULT, true);
		check_db_result($existing_configs, $sql, __LINE__, __FILE__);
		
		// add missing config values
		foreach ($new_dist_list_fields as $key => $value)
		{
			if (!isset($existing_configs[$key]))
			{
				$sql = "INSERT INTO " . _M2F_DISTRIBUTIONLIST_CONFIG_TABLE_ . "
									(distributionlist_id, config_name, config_value)
								VALUES ('$id', '$key', '$value')" ;
				$db_result = $m2f_db->query($sql);
				check_db_result($db_result, $sql, __LINE__, __FILE__);
			}
		}
	}    
	
	// Delete records which are no longer used
	$sql = "DELETE FROM " . _M2F_DISTRIBUTIONLIST_CONFIG_TABLE_ . "
						WHERE config_name = '_M2F_USE_ALL_ADDRESSES_'";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);
	
 	m2f_log(PEAR_LOG_NOTICE , "Updated distribution list settings" , __LINE__ , __FILE__);


	//
	// Adds new fields to tables
	//
	
	// Add the new logtime column to hash table - set all old records to time() (i.e. now)
	switch ($type_db)
	{
		case 'mysql':
			$sql = "ALTER TABLE " . _M2F_MESSAGE_HASH_TABLE_ . "
							ADD COLUMN (logtime int(10) NOT NULL default '0', id int(10) unsigned, tag char(3) default NULL)";
			break;

		case 'pgsql':
			$sql = "ALTER TABLE " . _M2F_MESSAGE_HASH_TABLE_ . "
							ADD COLUMN (logtime INTEGER NOT NULL default 0, id INTEGER, tag character(3) default NULL)";
			break;
	}
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);

	$sql = 'DESCRIBE ' . _M2F_MESSAGE_HASH_TABLE_;
	$db_result = $m2f_db->getAssoc($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);
	
	if (!empty($db_result['hash']['Key']) && $db_result['hash']['Key'] == 'PRI')
	{
		$sql = "ALTER TABLE " . _M2F_MESSAGE_HASH_TABLE_ . "
						DROP PRIMARY KEY";
		$db_result = $m2f_db->query($sql);
		check_db_result($db_result, $sql, __LINE__, __FILE__);
	}

	$sql = "ALTER TABLE " . _M2F_MESSAGE_HASH_TABLE_ . "
					ADD UNIQUE KEY hash (hash)";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);

	$sql = "UPDATE " . _M2F_MESSAGE_HASH_TABLE_ . "
					SET logtime = " . time();
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);

	$sql = "UPDATE " . _M2F_MESSAGE_HASH_TABLE_ . "
					SET tag = 'IMP'";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);





	// Update dist list table
	$sql = "ALTER TABLE " . _M2F_DISTRIBUTIONLIST_TABLE_ . "
					ADD COLUMN (visibility smallint(6) default '1')";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);
	
	
	
	// Update message_id table
	$sql = "ALTER TABLE " . _M2F_MESSAGE_ID_TABLE_ . "
					DROP PRIMARY KEY";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);

	$sql = "ALTER TABLE " . _M2F_MESSAGE_ID_TABLE_ . "
					ADD COLUMN (logtime int(10) NOT NULL default '0')";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);
	
	$sql = 'UPDATE ' . _M2F_MESSAGE_ID_TABLE_ . '
					SET logtime = \'' . time() . '\'';
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);


	switch ($type_db)
	{
		case 'mysql':
			// Change cron timestamp to logtime (int)
			$sql = "ALTER TABLE " . _M2F_CRON_TABLE_ . " CHANGE timestamp logtime int(10) NOT NULL";
			$db_result = $m2f_db->query($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);
		
			// Change dist_list_pattern field 'order' to 'pattern_order'
			$sql = "ALTER TABLE " . _M2F_DIST_LIST_PATTERNS_TABLE_ . " CHANGE `order` pattern_order int(11) NOT NULL default '0'";
			$db_result = $m2f_db->query($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);
		
			// Change pattern field 'regexp' to 'use_regexp'
			$sql = "ALTER TABLE " . _M2F_PATTERN_MATCH_TABLE_ . " CHANGE `regexp` use_regexp tinyint(1) NOT NULL default '0'";
			$db_result = $m2f_db->query($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);
		
			// Add column for purpose
			$sql = "ALTER TABLE " . _M2F_PATTERN_MATCH_TABLE_ . " ADD COLUMN (`purpose` varchar(6) NOT NULL default 'search')";
			$db_result = $m2f_db->query($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);
			break;

		case 'pgsql':
			// Change cron timestamp to logtime (int)
			$sql = "ALTER TABLE " . _M2F_CRON_TABLE_ . " CHANGE timestamp logtime INTEGER NOT NULL";
			$db_result = $m2f_db->query($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);
		
			// Change dist_list_pattern field 'order' to 'pattern_order'
			$sql = "ALTER TABLE " . _M2F_DIST_LIST_PATTERNS_TABLE_ . " CHANGE order pattern_order INTEGER NOT NULL default 0";
			$db_result = $m2f_db->query($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);
		
			// Change pattern field 'regexp' to 'use_regexp'
			$sql = "ALTER TABLE " . _M2F_PATTERN_MATCH_TABLE_ . " CHANGE regexp use_regexp INTEGER NOT NULL default 0";
			$db_result = $m2f_db->query($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);
		
			// Add column for purpose
			$sql = "ALTER TABLE " . _M2F_PATTERN_MATCH_TABLE_ . " ADD COLUMN (purpose VARCHAR(6) NOT NULL DEFAULT 'search')";
			$db_result = $m2f_db->query($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);
			break;
	}

 	m2f_log(PEAR_LOG_NOTICE , "Updated tables" , __LINE__ , __FILE__);

	
	// insert hardcoded Spam Detection patterns
	$sql = "INSERT INTO " . _M2F_PATTERN_MATCH_TABLE_ . " (pattern_match,pattern_replace,use_regexp,modifiers,description,target,internal,purpose)
					VALUES ('Your email requires verification','','0','','Block emails with \"Your email requires verification\" in Subject','Subject',1,'block')";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);
	
	$sql = "INSERT INTO " . _M2F_PATTERN_MATCH_TABLE_ . " (pattern_match,pattern_replace,use_regexp,modifiers,description,target,internal,purpose)
					VALUES ('Mail delivery failed: returning message to sender','','0','','Block emails with \"Mail delivery failed: returning message to sender\" in Subject','Subject',1,'block');";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);


	//
	// Create new tables
	//
	
	switch ($type_db)
	{
		case 'mysql':

			$sql = "CREATE TABLE " . _M2F_MODERATED_MSGS_TABLE_ . "
						(
							id int(10) NOT NULL default '0',
							distributionlist_id smallint(5) NOT NULL default '0',
							mailinglist_id smallint(5) NOT NULL default '0',
							forum_id smallint(5) NOT NULL default '0',
							topic_id smallint(8) NOT NULL default '0',
							user_id smallint(8) NOT NULL default '0',
							forum_msg longtext,
							mail_msg longtext,
							userdata text,
							recipient_list longtext,
							direction char(3),
							time int(10) NOT NULL default '0',
							PRIMARY KEY (id) 
						)"; 
			$db_result = $m2f_db->query($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);
		
			break;
			
		case 'pgsql':
		
			$sql = "CREATE TABLE " . _M2F_MODERATED_MSGS_TABLE_ . "
						(
							id INTEGER NOT NULL DEFAULT 0, 
							distributionlist_id INTEGER NOT NULL default 0,
							mailinglist_id INTEGER NOT NULL default 0,
							forum_id INTEGER NOT NULL default 0,
							topic_id INTEGER NOT NULL default 0,
							user_id INTEGER NOT NULL default 0,
							forum_msg TEXT,
							mail_msg TEXT,
							userdata TEXT,
							recipient_list TEXT,
							direction character(3),
							time INTEGER NOT NULL default 0,
							PRIMARY KEY (id) 
						)"; 
			$db_result = $m2f_db->query($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);

			break;
	}



	//
	// Add new / remove old config settings
	//

	$new_m2f_config_settings = array(
		'_M2F_LOG_PRUNE_' => '30',
		'_M2F_HASH_PRUNE_' => '30',
		'_M2F_DEFAULT_EMAIL_PREFS_' => '21'
		);
		
	foreach ($new_m2f_config_settings as $config_name => $config_value)
	{
		$sql = "INSERT INTO " . _M2F_CONFIG_TABLE_ . "
							(config_name, config_value)
						VALUES ('$config_name', '$config_value')";
		$db_result = $m2f_db->query($sql);
		check_db_result($db_result, $sql, __LINE__, __FILE__);
	}


	$old_m2f_config_settings = array(
		'_M2F_SEND_HTML_',
		'_M2F_RECONSTITUTE_HTML_MSGS_',
		'_M2F_SEND_ATTACHMENT_LINK_',
		);
		
	foreach ($old_m2f_config_settings as $config_name)
	{
		$sql = "DELETE FROM " . _M2F_CONFIG_TABLE_ . "
							WHERE config_name = '$config_name'";
		$db_result = $m2f_db->query($sql);
		check_db_result($db_result, $sql, __LINE__, __FILE__);
	}
	
 	m2f_log(PEAR_LOG_NOTICE , "Updated config settings" , __LINE__ , __FILE__);



	// remove old <<topic_link>> tag in signature if present
	$sql = "SELECT config_value FROM " . _M2F_CONFIG_TABLE_ . "
						WHERE config_name = '_M2F_MAIL_MSG_SIGNATURE_'";
	$old_sig = $m2f_db->getOne($sql);
	check_db_result($old_sig, $sql, __LINE__, __FILE__);

	$new_sig = preg_replace('/\<\<topic_link\>\>/X', '', $old_sig);
	
	if ($new_sig != $old_sig)
	{
		$sql = "UPDATE " . _M2F_CONFIG_TABLE_ . "
						SET config_value = '$new_sig'
						WHERE config_name = '_M2F_MAIL_MSG_SIGNATURE_'";
		$db_result = $m2f_db->query($sql);
		check_db_result($db_result, $sql, __LINE__, __FILE__);
	}

	
	
	//
	//  Alter phpBB table
	//
	
	switch ($type_db)
	{
		case 'mysql':
			$sql = "ALTER TABLE " . USERS_TABLE . "
							ADD COLUMN m2f_user_prefs int(1) NOT NULL DEFAULT '-1'";
			break;
			
		case 'pgsql':
			$sql = "ALTER TABLE " . USERS_TABLE . "
							ADD COLUMN m2f_user_prefs INTEGER NOT NULL DEFAULT '-1'";
			break;
	}
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);
		


	// Update version number
	$sql = "UPDATE " . _M2F_CONFIG_TABLE_ . "
	     		SET config_value = 9
	     		WHERE config_name = '_M2F_DB_VERSION_'";
	$db_result = $m2f_db->query($sql);
	check_db_result($db_result, $sql, __LINE__, __FILE__);
	m2f_log(PEAR_LOG_NOTICE , "Updated DB version to 9" , __LINE__ , __FILE__ );

	return TRUE;
}

?>