<?php
/***************************************************************************
 *                         functions_insert_post.php
 *                      -------------------------------
 *   Author  		: 	netclectic - Adrian Cockburn - adrian@netclectic.com
 *   Created 		: 	Monday, Sept 23, 2002
 *   Modified 		: 	Sunday, Feb 23, 2003
 *
 *	 File Version   :   1.0.6   -   fux bugs with special chars (this time :D)
 *                      1.0.5   -   add error_die_function parameter
 *                                  fux bugs with special chars
 *                                  remove check for version 
 *                      1.0.4   -   add current_time paramter to insert_post
 *                                  fixed bug with add_attach not including the functions_attach file
 *                      1.0.3   -   fixed small bug in insert_post - thanks to Pda0
 *	                 	1.0.2   -   added user_notification support
 *                      1.0.1   -   some bug fixes
 *                      1.0.0   -   original release
 ***************************************************************************/

/***************************************************************************
 *                             insert_post
 *                         -------------------
 *   Author         :   netclectic - Adrian Cockburn - adrian@netclectic.com
 *                          thanks to wineknow for the suggestion of adding the current_time parameter
 *   Version        :   1.0.6
 *   Created 		: 	Monday, Sept 23, 2002
 *   Last Updated   :   Thursday, March 27, 2003
 *
 *   Description    :   This functions is used to insert a post into your phpbb forums. 
 *                      It handles all the related bits like updating post counts, 
 *                      indexing search words, etc.
 *                      The post is inserted for a specific user, so you will have to 
 *                      already have a user setup which you want to use with it.
 *
 *   Parameters     :   $message            - the message that will form the body of the post
 *                      $subject            - the subject of the post
 *                      $forum_id           - the forum the post is to be added to
 *                      $user_id            - the id of the user for the post
 *                      $user_name          - the username of the user for the post
 *                      $user_attach_sig    - should the user's signature be attached to the post
 *
 *   Options Params :   $topic_id           - if topic_id is passed then the post will be 
 *                                              added as a reply to this topic
 *                      $post_id            - if post_id is passed then the 'phantom' post with this
  *                                              post_id will be updated
 *                      $topic_type         - defaults to POST_NORMAL, can also be
 *                                              POST_STICKY, POST_ANNOUNCE or POST_GLOBAL_ANNOUNCE
 *                      $do_notification    - should users be notified of new posts (only valid for replies)
 *                      $notify_user        - should the 'posting' user be signed up for notifications of this topic
 *                      $current_time       - should the current time be used, if not the you should supply a posting time
 *                      $error_die_function - 
 *                      $html_on = false    -
 *                      $bbcode_on = true   -
 *                      $smilies_on = true  -
 *
 *   Returns        :   If the function succeeds without an error it will return an array containing
 *                      the post id and the topic id of the new post. Any error along the way will 
 ***************************************************************************/
function m2f_insert_post (
    $message, 
    $subject, 
    $forum_id, 
    $user_id, 
    $user_name, 
    $user_attach_sig, 
    $topic_id = NULL, 
    $force_post_id = NULL,
    $topic_type = POST_NORMAL, 
    $do_notification = false, 
    $notify_user = false, 
    $current_time = 0, 
    $error_die_function = '', 
    $html_on = 0, 
    $bbcode_on = 1, 
    $smilies_on = 1 )
{
    global $db, $board_config;

		$user_ip = '00000000';
		
    // initialise some variables
    $topic_vote = 0; 
    $poll_title = '';
    $poll_options = '';
    $poll_length = '';
    $mode = 'reply'; 

    $bbcode_uid = ($bbcode_on) ? make_bbcode_uid() : ''; 
    $error_die_function = ($error_die_function == '') ? "message_die" : $error_die_function;
    $current_time = ($current_time == 0) ? time() : $current_time;
    
    // parse the message and the subject 
    $message = str_replace("\'", "''", prepare_message(trim($message), $html_on, $bbcode_on, $smilies_on, $bbcode_uid)); 
    $subject = str_replace("\'", "''", trim($subject)); 
    $username = str_replace("\'", "''", trim(strip_tags($user_name))); 
    
    // if this is a new topic then insert the topic details
    if ( is_null($topic_id) )
    {
        $mode = 'newtopic'; 
        $sql = "INSERT INTO " . TOPICS_TABLE . " (topic_title, topic_poster, topic_time, forum_id, topic_status, topic_type, topic_vote) VALUES ('$subject', " . $user_id . ", $current_time, $forum_id, " . TOPIC_UNLOCKED . ", $topic_type, $topic_vote)";
        if ( !$db->sql_query($sql, BEGIN_TRANSACTION) )
        {
            $error_die_function(GENERAL_ERROR, 'Error in posting', '', __LINE__, __FILE__, $sql);
        }
        $topic_id = $db->sql_nextid();
    }

		if ($force_post_id) // post_id set, therefore we should update a 'phantom' post rather than enter a new one
		{
			// update the post details, including the topic id
			$sql = "UPDATE " . POSTS_TABLE . " 
							SET topic_id = '$topic_id', forum_id = '$forum_id', poster_id = '$user_id', post_username = '$username', post_time = '$current_time', poster_ip = '$user_ip', enable_bbcode = '$bbcode_on', enable_html = '$html_on', enable_smilies = '$smilies_on', enable_sig = '$user_attach_sig'  
							WHERE post_id = '$force_post_id'";
			if ( !$db->sql_query($sql, BEGIN_TRANSACTION) )
			{
					$error_die_function(GENERAL_ERROR, 'Error in posting', '', __LINE__, __FILE__, $sql);
			}
			$post_id = $force_post_id;
		}
		else
		{
			// insert the post details using the topic id
			$sql = "INSERT INTO " . POSTS_TABLE . " (topic_id, forum_id, poster_id, post_username, post_time, poster_ip, enable_bbcode, enable_html, enable_smilies, enable_sig) VALUES ($topic_id, $forum_id, " . $user_id . ", '$username', $current_time, '$user_ip', $bbcode_on, $html_on, $smilies_on, $user_attach_sig)";
			if ( !$db->sql_query($sql, BEGIN_TRANSACTION) )
			{
					$error_die_function(GENERAL_ERROR, 'Error in posting', '', __LINE__, __FILE__, $sql);
			}
			$post_id = $db->sql_nextid();
		}
    
    // insert the actual post text for our new post
    $sql = "INSERT INTO " . POSTS_TEXT_TABLE . " (post_id, post_subject, bbcode_uid, post_text) VALUES ($post_id, '$subject', '$bbcode_uid', '$message')";
    if ( !$db->sql_query($sql, BEGIN_TRANSACTION) )
    {
        $error_die_function(GENERAL_ERROR, 'Error in posting', '', __LINE__, __FILE__, $sql);
    }


		// find out if this post is actually the newest of the topic and/or forum
		// or if it is older than some existing posts (quirk of m2f - posts can be made out of order)


		// check date of last post in forum
		$sql = "SELECT p.post_time
							FROM " . POSTS_TABLE . " p, " . FORUMS_TABLE . " f 
							WHERE f.forum_last_post_id = p.post_id
							LIMIT 1";
		if ( !$db->sql_query($sql, BEGIN_TRANSACTION) )
		{
				$error_die_function(GENERAL_ERROR, 'Error in posting', '', __LINE__, __FILE__, $sql);
		}
		$row = $db->sql_fetchrow($result);
		$last_post_forum_time = $row['post_time'];
	
		if ($mode == 'newtopic')
		{
			// update the post counts etc.
			$lastpostsql = ($current_time > $last_post_forum_time) ? ",forum_last_post_id = $post_id" : ''; // is this post now the newest in the forum?
			$sql = "UPDATE " . FORUMS_TABLE . " SET 
									forum_posts = forum_posts + 1,
									forum_topics = forum_topics + 1
									$lastpostsql 	
							WHERE forum_id = $forum_id";
			if ( !$db->sql_query($sql, BEGIN_TRANSACTION) )
			{
					$error_die_function(GENERAL_ERROR, 'Error in posting', '', __LINE__, __FILE__, $sql);
			}
	
			// update the first / last post ids for the topic
			$sql = "UPDATE " . TOPICS_TABLE . " SET 
									topic_last_post_id = $post_id, 
									topic_first_post_id = $post_id
							WHERE topic_id = $topic_id";
			if ( !$db->sql_query($sql, BEGIN_TRANSACTION) )
			{
					$error_die_function(GENERAL_ERROR, 'Error in posting', '', __LINE__, __FILE__, $sql);
			}
		}
		else
		{
			// update the post counts etc.
			$lastpostsql = ($current_time > $last_post_forum_time) ? ",forum_last_post_id = $post_id" : ''; // is this post now the newest in the forum?
			$sql = "UPDATE " . FORUMS_TABLE . " SET 
									forum_posts = forum_posts + 1
									$lastpostsql 	
							WHERE forum_id = $forum_id";
			if ( !$db->sql_query($sql, BEGIN_TRANSACTION) )
			{
					$error_die_function(GENERAL_ERROR, 'Error in posting', '', __LINE__, __FILE__, $sql);
			}
	
			// check date of last post in topic
			$sql = "SELECT p.post_time
								FROM " . POSTS_TABLE . " p, " . TOPICS_TABLE . " t 
								WHERE t.topic_last_post_id = p.post_id
								LIMIT 1";
			if ( !$db->sql_query($sql, BEGIN_TRANSACTION) )
			{
					$error_die_function(GENERAL_ERROR, 'Error in posting', '', __LINE__, __FILE__, $sql);
			}
			$row = $db->sql_fetchrow($result);
			$last_post_topic_time = $row['post_time'];
		
			// update the first / last post ids for the topic
			$lastpostsql = ($current_time > $last_post_forum_time) ? ",topic_last_post_id = $post_id" : ''; // is this post now the newest in the forum?
			$sql = "UPDATE " . TOPICS_TABLE . " SET 
									topic_replies=topic_replies+1
									$lastpostsql
							WHERE topic_id = $topic_id";
			if ( !$db->sql_query($sql, BEGIN_TRANSACTION) )
			{
					$error_die_function(GENERAL_ERROR, 'Error in posting', '', __LINE__, __FILE__, $sql);
			}
		}

    // update the user's post count and commit the transaction
    $sql = "UPDATE " . USERS_TABLE . " SET 
                user_posts = user_posts + 1
            WHERE user_id = $user_id";
    if ( !$db->sql_query($sql, END_TRANSACTION) )
    {
        $error_die_function(GENERAL_ERROR, 'Error in posting', '', __LINE__, __FILE__, $sql);
    }
    
    // add the search words for our new post
    switch ($board_config['version'])
    {
        case '.0.0' : 
        case '.0.1' : 
        case '.0.2' : 
        case '.0.3' : 
            add_search_words($post_id, stripslashes($message), stripslashes($subject));
            break;
        
        default :
            add_search_words('', $post_id, stripslashes($message), stripslashes($subject));
            break;
    }
    
    // do we need to do user notification
    // do this even for new topics, so that an entry will be put in the watch table (if needed)
    if ( $do_notification )
    {
        $post_data = array();
        
        // Declare and define $userdata. The user_notification function uses this to set the
        // topics_watch table.
        global $userdata;
        $userdata['user_id'] = $user_id;
        
        user_notification($mode, $post_data, $subject, $forum_id, $topic_id, $post_id, $notify_user);
    }
    
    // if all is well then return the id of our new post
    return array('post_id'=>$post_id, 'topic_id'=>$topic_id, 'bbcode_uid'=>$bbcode_uid, 'message'=>$message);
}

?>