<?php


/**
  * m2f_attachments.php
  *
  * Author Nick Penna
  *
  * Version 0.4.0
*/





function is_db_error($db_result, $sql, $line, $file)
{
  if (DB::isError($db_result))
  {
    $m2f_error_msg = "DB error. sql: '$sql' error: " . $db_result->getMessage();
    m2f_log(PEAR_LOG_CRIT , $m2f_error_msg , $line, $file);
    return true;
  }
  return false;
}





class attachment_details
{
/*--------------------------------------------------------------------------------------------------
																									
								CLASS attachment_details											

								------------------------											

																									

 Used to hold the details for, and process one attachment, both inbound and outbound.				
 The m2f_attachments class holds an array of these.												
																									

 All calls are handled through m2f_attachments														
																									

 For inbound emails:																				
	1)  An instance is created when a raw email has an attachment. The mime part of the attachment	
	    is copied into $mimepart.																	
	2)  Before the email is saved, function validate_mime() is called to check details like 
		filename, extension permission, forum permissions etc. During this $filename, $size & $mime
		get populated.
	3)	Function write_to_disk creates the physical file and populates $physical_name
	4)	After the post is saved to the database, insert_to_db() adds details of the attachment.
	5)	When the corresponding outbound mail is sent,add_outgoing($msg) adds to $msg the attachment as 
		a mime part. $msg must be a reference and $physical_name must be populated.
		
 For emails generated by a posting:
 	1)	An instance is created on the forum_msg with $physical_name, $filename and $comment
 	2) 	The instance is copied to $mail_msg when m2f_attachments is copied.
 	3)  When the outbound mail is sent the processing is as for (5) above.		
---------------------------------------------------------------------------------------------------*/


	var 	$filename, 			// original filename from either the post or an inbound email
			$physical_name,		// name used to save the file, modified if the filename is already used.
			$extension,			// file extension
			$size,				// file size
			$mime,				// mime encoding
			$filetime,			// time file was saved
			$comment,			// from post
			$mimepart,			// mime part from inbound email
			$thumbnail,			// do we have to create a thumbnail or not
			$attach_id;			// database id in attachments_desc for use with download.php

//---------------------------------------------------------------------------------------------------
	
	// Constructor
	function attachment_details()
	{
		$this->thumbnail = 0;
	}


//---------------------------------------------------------------------------------------------------
	function is_image()
	{
		// Checks if this attachment is an image
		return  (((strlen($this->mime) > 5) && (strtolower(substr($this->mime, 0, 5)) == 'image')));
	}


//---------------------------------------------------------------------------------------------------
	function insert_to_db($post_id, $user_id)
	{
		global $m2f_db;
		//
		// insert attachment into db 
		//
		$error_msg = "Your email could not be processed due to technical problems. please contact the board Administrator.";
		
		$sql = "INSERT INTO " . ATTACHMENTS_DESC_TABLE . " (physical_filename, real_filename, comment, extension, mimetype, filesize, filetime, thumbnail) 
			VALUES ( '" . str_replace("'", "''", basename($this->physical_name)) . "', '" . str_replace("'", "''", basename($this->filename)) . "', '', '" . $this->extension . "', '" . $this->mime . "', " . $this->size . ", " . $this->filetime . ", " . $this->thumbnail . ")";

		$result = $m2f_db->query($sql);
		if (is_db_error($result, $sql, __LINE__, __FILE__))
		{
			return $error_msg;
		}
					
			
		// get back the attach_id - This PEAR implementation seems not to have ->next_id()
		$sql = "SELECT attach_id FROM " . ATTACHMENTS_DESC_TABLE . " WHERE physical_filename = '" . str_replace("'", "''", basename($this->physical_name) ) . "'";
	
		$result = $m2f_db->getOne($sql);
		if (is_db_error($result, $sql, __LINE__, __FILE__))
		{
			return $error_msg;
		}
				
		$this->attach_id = $result;
	
		// insert link into attachments table			
		$sql = 'INSERT INTO ' . ATTACHMENTS_TABLE . ' (attach_id, post_id, privmsgs_id, user_id_1, user_id_2)' .
				' VALUES (' . $this->attach_id . ', ' . $post_id . ', 0, ' . $user_id . ', 0)';
				
		$result = $m2f_db->query($sql);
		if (is_db_error($result, $sql, __LINE__, __FILE__))
		{
			return $error_msg;
		}

		$error_msg = "";
		return $error_msg;		
	} //insert_to_db
	
//---------------------------------------------------------------------------------------------------
	
	function validate_mime($forum_id, $userdata)
	{
	//
	// When called, the mimepart must already be present.
	// This function extracts all the data from the mimepart into the other variables
	// Does lots of checking 
	//
		global $m2f_db;
		global $attach_config;

		$error_msg = "";
		
		if(!isset($this->mimepart))
		{
			$error_msg = "Error in [save_inbound_to_disk()] mimepart not set!";
			m2f_log(PEAR_LOG_CRIT , $m2f_error_msg , $line, $file);
			$error_msg = "Your email could not be processed due to technical problems. please contact the boar Administrator.";
			return $error_msg;
		}

		// extract filename
		$this->filename = $this->mimepart->d_parameters['filename'];
		// no filename? try name instead
		if ($this->filename == "")
		{
			$this->filename = $this->mimepart->d_parameters['name'];
		}	 	
		// still no filename? try ctype_parameters name instead
		if ($this->filename == "")
		{
			$this->filename = $this->mimepart->ctype_parameters['name'];
		}	 			
				
		if ($this->filename == "") {
			$error_msg = "Attachment must have a filename to be saved to a forum.";
			return $error_msg;
		}
		
		$this->filename = str_replace(' ', '_', $this->filename); 	
		if ( preg_match("/[\\/:*?\"<>|]/i", $this->filename) )
		{ 
			$error_msg = 'The filename for the attachment was invalid [' . $this->filename .']';
			return $error_msg;
		}
			
		// get size	
		$this->size = strlen($this->mimepart->body);
			
		// get extension
		$this->extension = get_extension($this->filename);
		if ($this->extension == "") {
			$error_msg = "Attachment must have a valid extension to be saved to a forum.";
			return $error_msg;
		}

		// is extension allowed?	
		$sql = "SELECT g.allow_group, g.max_filesize, g.cat_id, g.forum_permissions
				FROM " . EXTENSION_GROUPS_TABLE . " g, " . EXTENSIONS_TABLE . " e
				WHERE (g.group_id = e.group_id) AND (e.extension = '" . $this->extension . "')
				LIMIT 1";
					
		$db_result = $m2f_db->getRow($sql);
		if (is_db_error($db_result, $sql, __LINE__, __FILE__))
		{
			$error_msg = "Your email could not be processed due to technical problems. please contact the board Administrator.";
			return $error_msg;   		
		}
		$row = $db_result;
		
		if (intval($row['allow_group']) == 0)
		{
			$error_msg = 'The extension [' . $this->extension . '] is not allowed.';
			return $error_msg;
		} 
			
	
		//
		// Check Forum Permissions
		//
		$auth_cache = trim($row['forum_permissions']);		
		if (  ($userdata['user_level'] != ADMIN) && (!is_forum_authed($auth_cache, $forum_id) && (trim($auth_cache) != '')) )
		{
			$error_msg = 'Files with the supplied extension [ '. $this->extension . ' are not permitted in this Forum'; 
			return $error_msg;
		} 

		//
		// check Filesize 
		//
		$allowed_filesize = ($row['max_filesize']) ? $row['max_filesize'] : $attach_config['max_filesize'];
		$cat_id = intval($row['cat_id']);

		if  (($allowed_filesize != 0) && ($this->size > $allowed_filesize) && ($userdata['user_level'] != ADMIN) )
		{
			$size_lang = ($allowed_filesize >= 1048576) ? 'MB' : ( ($allowed_filesize >= 1024) ? 'KB' : 'Bytes' );

			if ($allowed_filesize >= 1048576)
			{
				$allowed_filesize = round($allowed_filesize / 1048576 * 100) / 100;
			}
			else if($allowed_filesize >= 1024)
			{
				$allowed_filesize = round($allowed_filesize / 1024 * 100) / 100;
			}

			$error_msg = 'An attachment is too big. Maximum is ' .  $allowed_filesize . ' ' . $size_lang;
			return $error_msg; 
		}
		
		//
		// Check Image dimensions
		//
		$cat_id = intval($row['cat_id']);
		if ( function_exists('imagecreatefromstring') && ($userdata['user_level'] != ADMIN) && ($cat_id == IMAGE_CAT) )
		{
			$img = imagecreatefromstring($this->mimepart->body);
			$width = imagesx($img);
			$height = imagesy($img);

			if ( ($width != 0) && ($height != 0) && (intval($attach_config['img_max_width']) != 0) && (intval($attach_config['img_max_height']) != 0) )
			{
				if ( ($width > intval($attach_config['img_max_width'])) || ($height > intval($attach_config['img_max_height'])) )
				{
					$error_msg = 'Attached image exceeds dimension limits of ' . intval($attach_config['img_max_width']) . "wide  by " . intval($attach_config['img_max_height'] . ' tall');
					return $error_msg;
				}
			}
		}		
				
		if ( ($cat_id == IMAGE_CAT) && (intval($attach_config['img_create_thumbnail'])) )
		{
			$this->thumbnail = 1;
		}		
				
				
		// copy mime details
		$this->mime = $this->mimepart->ctype_primary  . '/' . $this->mimepart->ctype_secondary;
		return $error_msg;
	} // function validate_mime
	
//---------------------------------------------------------------------------------------------------

	 function write_to_disk()			
	 {
		global $upload_dir;
		global $phpbb_root_path;

		$error_msg = "";
		
		
		$disk_filename = rawurlencode($this->filename);
		$disk_filename = preg_replace("/%(\w{2})/", "_", $disk_filename);
		$disk_filename = delete_extension($disk_filename);
	
		if (is_dir($upload_dir))
		{
			// probably running from board
			$directory =  $upload_dir . '/';
		} else
		{
			$directory = $phpbb_root_path . $upload_dir . '/';
		}
		do
		{
			$this->physical_name = $disk_filename . '_' . substr(rand(), 0, 3) . '.' . $this->extension;
			// adjust for script location
			
			$target_filename = $directory . $this->physical_name;
		}
		while (file_exists($target_filename));
				
    	
    	//save the file      
        // open file
        $fp = @fopen($target_filename, 'w');
        		
        if ($fp == FALSE) {
        		$error_msg = "Unable to save attachment [" . $target_filename . "]";
        		return $error_msg;
      	}		
        else { 
    		// write body
    		if ( fwrite($fp, $this->mimepart->body) === FALSE)
    		{
    			$error_msg = "Unable to write attachment [" . $target_filename . "]";
    		}
   			// close file
   			m2f_log(PEAR_LOG_INFO, "Attachment saved: " . $this->filename . " as " . $this->physical_name, __LINE__ , __FILE__ );
   			fclose($fp);
   			$this->filetime = time();
		}
		
		if ($this->thumbnail){
			if ($error_msg <> ""){
				$this->thumbnail = 0;
			} else
			{
				$thumb_file = $directory .  THUMB_DIR . '/t_' . $this->physical_name;
				
				if (!create_thumbnail($target_filename, $thumb_file, $this->mime))
				{
				 	$this->thumbnail = 0;
				}
			}		
		}
		
		return $error_msg;
	}
	
//---------------------------------------------------------------------------------------------------
		
	function download_link()
	{
		global $board_config;
		global $upload_dir;
		global $m2f_db;
			
		$server_name = trim($board_config['server_name']);
		$server_protocol = ($board_config['cookie_secure']) ? 'https://' : 'http://';
		$server_port = ($board_config['server_port'] <> 80) ? ':' . trim($board_config['server_port']) . '/' : '/';
		//$script_path = trim($board_config['script_path']);
		$script_path = preg_replace('/^\/?(.*?)\/?$/', '\1', trim($board_config['script_path'])) . '/';

		
		//$link_style = _M2F_ATTACHMENT_LINK_STYLE_;
		$link_style = 0;
		
		switch ($link_style) {
	
		case 1:
   			// download.php
   			$url = $server_protocol . $server_name . $server_port . $script_path . "download.php?id=". $this->attach_id;
   			break;
   			
   		case 2:
   			// login.php?redirect=download.php?id=
		   			
   			$url = $server_protocol . $server_name . $server_port . $script_path . "login.php?redirect=download.php?id=". $this->attach_id;
   			break;
   		
		default:
     		// url to filename
   			$url = $server_protocol . $server_name . $server_port . $script_path .  $upload_dir . '/' . $this->physical_name;
   			break;
  		}
  		return $url;
	}

		
	
	
//---------------------------------------------------------------------------------------------------
		
	function html_link()
	{
		
		if (isset($this->comment) && ($this->comment <>""))
		{
			$caption = stripslashes($this->comment);
		} 
		else
		{
			$caption = $this->filename;
		}
		
		$link = '<a href = "' . $this->download_link() . '">' . $caption . '</a><br />';
		m2f_log(PEAR_LOG_DEBUG, "Adding html attachment link: " . $this->filename, __LINE__ , __FILE__ );
		
		return $link; 
	}	
			
//---------------------------------------------------------------------------------------------------

	// Create html for inline images, to be appended to html_body 
		
	function inline_html_link($thumbnail_images)
	{
		// Check if we have an image. Only images get inlined	
		if ($this->is_image()){
		
			// Calculate a comment
			if (isset($this->comment) && ($this->comment <>""))
			{
				$caption = $this->comment;
			} 
			else
			{
				$caption = str_replace('.', ' ',$this->filename);
			}
			
			// Generate a table to put the image in
			$html = '<div align="center"><hr width="95%" /></div>';
			$html .= '<table width="95%" border="1" cellpadding="2" cellspacing="0" align="center" class="attachtable">';
			$html .= '<tr>';
			$html .= '<td width="100%" class="attachheader" align="center"><span class="gen">' . $caption . '</span></td>';			
			$html .= '</tr>';
			
			// add header for thumbnails
			if (($thumbnail_images) && $this->thumbnail)
			{
				$html .= '<tr>';
				$html .= '<td width="100%" align="center">Click on thumbnail for full size image</td>';			
				$html .= '</tr>';
				$link_prefix = '<a href="' . $this->download_link() . '">';
				$link_postfix = '</a>'; 
			}
			
			// Insert the image			
			$html .= '<tr>';
			$html .= '<td colspan="2" align="center"><br />' . $link_prefix .'<img src="' . $this->filename . '" alt="'. $caption . '" border="0" />' . $link_postfix . '<br /><br /></td>';
			$html .= '</tr>';

			$html .= '</table>';
			$html .= '<div align="center"><hr width="95%" /></div>';
			
			m2f_log(PEAR_LOG_DEBUG, "Adding html attachment inline link: " . $this->filename, __LINE__ , __FILE__ );
		}
		return $html; 
	}	

//---------------------------------------------------------------------------------------------------

	function add_outgoing($msg, $inline_messages, $thumbnail_images)
	{
		global $upload_dir;
		global $phpbb_root_path;

		// do we already have the mimepart loaded i.e. from inbound mail AND it isn't thumbnailed?
		if (isset($this->mimepart) && (($this->thumbnail == 0) || ( $thumbnail_images = false)))
		{
			if (($inline_messages) && ($this->is_image()))
			{
				$msg->addHTMLImage($this->mimepart->body, $this->mime, $this->filename, FALSE );
			} 
			else
			{
				$msg->addAttachment($this->mimepart->body, $this->mime, $this->filename, FALSE );
			}
		}
		else
		{
			//look for the directory
			if (!is_dir($upload_dir))
			{
				// probably not running from board
				$upload_dir = $phpbb_root_path  . $upload_dir;
			}
			
			//thumbnail ??
			$thumb_filename = $upload_dir . '/' . THUMB_DIR . '/t_' . $this->physical_name;

			if (($this->thumbnail) && file_exists( $thumb_filename) && ($thumbnail_images))
			{
				$full_filename = $thumb_filename;
			} else
			{
				$full_filename = $upload_dir . '/' . $this->physical_name;
			}

			// open and read in the file			
			$fp = @fopen($full_filename, "rb");

			if ($fp <> FALSE) 
			{
				$contents = @fread($fp, filesize($full_filename));
				@fclose($fp);
			}
				
			if (strlen($contents) > 0)
			{ 
				if (($inline_messages) && ($this->is_image()))
				{
					$msg->addHTMLImage($contents, $this->mime, $this->filename, FALSE );
				} 
				else
				{
					$msg->addAttachment($contents, $this->mime, $this->filename, FALSE );
				}
				m2f_log(PEAR_LOG_INFO, "Adding outbound attachment: " . $this->filename, __LINE__ , __FILE__ );
			}
		}	
	}
}		// end of class

//---------------------------------------------------------------------------------------------------

class m2f_attachments
{
/*--------------------------------------------------------------------------------------------------
																									
								CLASS m2f_attachments											
								---------------------											
																									
 Used to hold and manage attachments.				
 Each attachment is an instance of attachment_details in the array $attach_data.												
																									
 For inbound emails:			
 	1) 	An instance is created on $mail_msg by the mail_msg constructor.																	
	2)  copy_from_inbound recurses through the mime structure to find images & attachments. For each
		one an attachment_details is added to $attach_data. And the actual part saved to	
	    attachment_details->$mimepart.																	
	3)  Before the email is saved, function validate_and_save_inbound() is called to check details of 
		the attachments and write them to disk. Most of the checking, other than quotas, is delegated
		to attachment_details. Writing is also delegated. 
	4)	After the post is written to the database,  function update_post handles all the database 
		updates. Updating the attachment details is delegated. Function mark_post_and_topic sets
		flags on the post and topic records to indicate attachments present.
	5)	When the corresponding outbound mail is sent, one or more of the following functions 
		are used:
			a) 	text_links() returns links to each of the file for text only emails.
			b) 	html_links() returns links to each of the file for html emails if a 'links' rather than 
				attachments is needed.
			c) 	add_outgoing($msg) adds to $msg the attachments as a mime parts. $msg must be a 
				reference.
		
 For emails generated by a posting:
 	1)	An instance is created on the forum_msg by the forum_msg constructor
 	2) 	Function grab_post_attachments is called to pull in details of attachments from the post.
 		For each one an instance of m2f_attachments is added to $attach_data.
 	3)	The instance is copied from forum_msg to mail_msg	
 	4)  When the outbound mail is sent the processing is as for (5) above.		
---------------------------------------------------------------------------------------------------*/

	var $attach_data;

	
	function m2f_attachments( )
	{

	}

	function mark_post_and_topic( $post_id, $topic_id)
	{
		global $m2f_db;
		
		// update posts table		
		$sql = "UPDATE " . POSTS_TABLE . " SET post_attachment = 1 	WHERE post_id = " . $post_id;
		$result = $m2f_db->query($sql);
		if (!is_db_error($result, $sql, __LINE__, __FILE__))
		{
			// update topic	
			$sql = "UPDATE " . TOPICS_TABLE . " SET topic_attachment = 1 WHERE topic_id = " . $topic_id;
			$result = $m2f_db->query($sql);
			$ignore = is_db_error($db_result, $sql, __LINE__, __FILE__);
		}
		
	}

//---------------------------------------------------------------------------------------------------

	function update_post($post_id, $topic_id, $user_id)
	{
		//
    	//	Save in database
    	//
        	
    	if (count($this->attach_data) > 0 )
    	{
    	
    		for ($i = 0; $i < count($this->attach_data); $i++)
			{
	    		$this->attach_data[$i]->insert_to_db($post_id, $user_id);
			}
			// set indicator on post and topic
    		$this->mark_post_and_topic( $post_id, $topic_id);
    	}
    }

//---------------------------------------------------------------------------------------------------
    
	function validate_and_save_inbound_attachments($userdata, $forum_id)
	{
		global  $attach_config;
		global $m2f_db;

		$error_msg = "";	

		if (count($this->attach_data) < 1 )
		{
			return $error_msg;
		}
		
		// Exceeds maximum attachments?
		if (($attach_config['max_attachments'] <> 0) && ($attach_config['max_attachments'] < count($this->attach_data)))
		{
			$error_msg = "The number of attachments in this email exceeds the maximum number of " . $attach_config['max_attachments'];
			m2f_log(PEAR_LOG_NOTICE , "Attach error: " . $error_msg , __LINE__ , __FILE__ );
			return $error_msg; 
		}
		
		for ($i = 0; $i < count($this->attach_data); $i++)
		{
	    	$error_msg = $this->attach_data[$i]->validate_mime($forum_id, $userdata);
	    	if ($error_msg <> "")
	    	{
	    	  m2f_log(PEAR_LOG_NOTICE , "Attach error: " . $error_msg , __LINE__ , __FILE__ );
	    		return $error_msg;
	    	}

		}
		

		for ($i = 0; $i< count($this->attach_data); $i++) 
		{ 
			if(strtolower(basename($this->attach_data[$i]->filename)) == "winmail.dat") 
			{ 
				unset($this->attach_data[$i]); 
			} 
		} 
		
		
		//TODO Quota checking like attach mod
		
		//
		// Check our complete quota
		//
		//$quota = $this->m2f_get_quota_limits($userdata['user_id']);
		
		
		$total_attach_size = 0;
		for ($i = 0; $i < count($this->attach_data); $i++)
		{
	    	$total_attach_size += $this->attach_data[$i]->size;
	    }		

		if ($attach_config['attachment_quota'])
		{
			$sql = 'SELECT sum(filesize) as total FROM ' . ATTACHMENTS_DESC_TABLE;

			
			$db_result = $m2f_db->getRow($sql);
    		if(is_db_error($db_result, $sql, __LINE__, __FILE__))
    		{
    			$error_msg = "Your email could not be processed due to technical problems. please contact the Board Administrator.";
    			m2f_log(PEAR_LOG_NOTICE , "Attach error: " . $error_msg , __LINE__ , __FILE__ );
				return $error_msg;   
    		}

			$row = $db_result;				
			$total_filesize = $row['total'];
			if (($total_filesize + $total_attach_size) > $attach_config['attachment_quota'])
			{
				$error_msg = 'Sorry, but the maximum filesize for all Attachments is reached. Please contact the Board Administrator if you have questions.';
				m2f_log(PEAR_LOG_NOTICE , "Attach error: " . $error_msg , __LINE__ , __FILE__ );
				return $error_msg;
			}
		
		}
		
		//
		// ok so far so write to disk
		//
		
		for ($i = 0; $i < count($this->attach_data); $i++)
		{
	    	$error_msg = $this->attach_data[$i]->write_to_disk();
	    	if ($error_msg <> "")
	    	{
	    	    m2f_log(PEAR_LOG_NOTICE , "Attach error: " . $error_msg , __LINE__ , __FILE__ );
	    		return $error_msg;
	    	}
		}		
	}
		
//---------------------------------------------------------------------------------------------------

	function add( &$part ) 
	{ 
		$newattach_data = new attachment_details; 
		$newattach_data->mimepart = $part; 
		$this->attach_data[] =  $newattach_data; 
		m2f_log(PEAR_LOG_INFO, "Adding attachment: Disp: " . $part->disposition . " Type: " . $part->ctype_primary, __LINE__ , __FILE__ ); 
	}

//---------------------------------------------------------------------------------------------------

	function copy_from_inbound($subparts)	
	{	
		// recursive function to extract attachments
		if (isset($subparts)) {
			if(!is_array($subparts))
			{
				m2f_log(PEAR_LOG_NOTICE , "Attach error: malformed MIME part."  , __LINE__ , __FILE__ );
				return;
			}
			foreach ($subparts as $part)
			{
				m2f_log(PEAR_LOG_DEBUG, "Processing subpart Disp: " . $part->disposition . " Type: " . $part->ctype_primary, __LINE__ , __FILE__ );
				// is it a part of interest?
				// must either be an attachment OR be inline and not text/html
    			if (
    					isset($part->disposition) and (strtolower($part->disposition)=='attachment') 
							or ((strtolower($part->disposition)=='inline') and (strtolower($part->ctype_primary) <> 'text'))
							or (strtolower($part->ctype_primary) == 'image')
							)
				{
					$newattach_data = new attachment_details;
					$newattach_data->mimepart = $part;
					$this->attach_data[] =  $newattach_data;
					m2f_log(PEAR_LOG_INFO, "Adding attachment: Disp: " . $part->disposition . " Type: " . $part->ctype_primary, __LINE__ , __FILE__ );
				}
				else if (strtolower($part->ctype_primary) == 'multipart')
				{
					$this->copy_from_inbound( $part->parts);
				}
				else if ((strtolower($part->ctype_primary) == 'message') && (strtolower($part->ctype_secondary) == 'rfc822'))
				{
					$this->copy_from_inbound( $part->parts);
				}
			}
		}
	}		

//---------------------------------------------------------------------------------------------------
    	
   function add_outgoing(&$msg, $inline_images = false, $thumbnail_images = true)
	{
		for ($i = 0; $i < count($this->attach_data); $i++)
		{
				$this->attach_data[$i]->add_outgoing(&$msg, $inline_images, $thumbnail_images);
		}
	}

//---------------------------------------------------------------------------------------------------


	function inline_html($thumbnail_images)
	{
		$links = "";
		//$links = '<br />Attachments <br />';
		for ($i = 0; $i < count($this->attach_data); $i++)
		{
			$links .= $this->attach_data[$i]->inline_html_link($thumbnail_images);
		}
		//$links .= '<br />';
		return $links ? '<br />Attachments <br />' . $links . '<br />' : $links;
	}


//---------------------------------------------------------------------------------------------------

	function html_links()
	{
		$links = ($num = count($this->attach_data)) ? '<br />Attachments:<br />' : '';
		for ($i = 0; $i < $num; $i++)
		{
			$links .= $this->attach_data[$i]->html_link();
		}
		if (!empty($links)) $links .= '<br />';
		return $links;
	}

//---------------------------------------------------------------------------------------------------

	function attach_count()
	{
		return count($this->attach_data);
	}
	
//---------------------------------------------------------------------------------------------------

	function requires_multipart($prefs)
	{
		// specified whether a mime multipart email is required according to the attachments and user prefs 
		// user has specifed html OR 
		//	there are attachments 
		//		and user has not requested links 
		//		and has not said ignore attachments
		return (($prefs & _M2F_USER_HTML_EMAIL_) || 
				(	($this->attach_count() > 0) 
					&& (($prefs & _M2F_USER_ATTACHMENT_LINKS_) == 0)  
					&& (($prefs & _M2F_USER_IGNORE_ATTACHMENTS_) == 0)));
	}
//---------------------------------------------------------------------------------------------------

	function image_count()
	{
				
		$images = 0;
		if ($this->attach_count())
		{
			for ($i = 0; $i < count($this->attach_data); $i++)
			{
				if ($this->attach_data[$i]->is_image()){
					$images = $images + 1;
				}
			}
		}
		
		return $images;
	}

//---------------------------------------------------------------------------------------------------

	function text_links()
	{
				 
		if (count($this->attach_data) > 0)
		{ 
			$links = "\nAttachments: \n";
			for ($i = 0; $i < count($this->attach_data); $i++)
			{
				if ($this->attach_data[$i]->comment <> "")
					{
						$message .=  $this->attach_data[$i]->comment . "\n";
						m2f_log(PEAR_LOG_DEBUG, "Adding attachment link to text: " . $this->filename, __LINE__ , __FILE__ );
					}
				$links .= $this->attach_data[$i]->download_link() . "\n";
			}
		}
		$links .= "\n";
		return $links;
	}

//---------------------------------------------------------------------------------------------------
	
	function html_end($user_prefs)
	{
		// create an appropriate html piece to tack at the end of the html_body
		// assume function requires_multipart (above) has already been checked.
		// choices are:
   		//      HTML Links
   		//		FILE attachments
   		//		IMAGES inline + non images as attachments

		$html = "";
		
		if ( !($user_prefs & _M2F_USER_IGNORE_ATTACHMENTS_) )
		{
			// it it just links ?	
			if ($user_prefs & _M2F_USER_ATTACHMENT_LINKS_)
			{
				$html = $this->html_links();
			} 
			// or inline attachments
			else if ($user_prefs & _M2F_USER_IMAGES_INLINE_) 
			{ 
				$html = $this->inline_html(($user_prefs & _M2F_USER_THUMBNAIL_IMAGES_));
			}
			
			$html .= "</body>";
		}
		
		return $html;
	}


//---------------------------------------------------------------------------------------------------
	
	function get_post_attachments()
	{
		global $attachment_mod;
	
		if (isset($attachment_mod['posting']))
		{
			for ($i = 0; $i < count($attachment_mod['posting']->attachment_list); $i++)
			{
				$new_attach = new attachment_details;
			
				$new_attach->physical_name = basename($attachment_mod['posting']->attachment_list[$i]);
				$new_attach->filename = $attachment_mod['posting']->attachment_filename_list[$i];
				$new_attach->comment =  $attachment_mod['posting']->attachment_comment_list[$i] ;
				$new_attach->mime =  $attachment_mod['posting']->attachment_mimetype_list[$i];
				$new_attach->thumbnail =  $attachment_mod['posting']->attachment_thumbnail_list[$i];
				$new_attach->attach_id =  $attachment_mod['posting']->attachment_id_list[$i];
				$this->attach_data[] = $new_attach; 
			}
		}
	}	
		
//---------------------------------------------------------------------------------------------------		
// NOT USED AS YET
	 function m2f_get_quota_limits($user_id )
	{
		global $attach_config, $m2f_db;

	//
	// Define Filesize Limits (Prepare Quota Settings)
	// Priority: User, Group, Management
	//
	// This method is somewhat query intensive, but i think because this one is only executed while attaching a file, 
	// it does not make much sense to come up with an new db-entry.
	// Maybe i will change this in a future version, where you are able to disable the User Quota Feature at all (using
	// Default Limits for all Users/Groups)
	//

	// Change this to 'group;user' if you want to have first priority on group quota settings.
	// 	$priority = 'group;user';
		$priority = 'user;group';


		$quota_type = QUOTA_UPLOAD_LIMIT;
		$limit = 0;
		$priority = explode(';', $priority);
		$found = FALSE;

		for ($i = 0; $i < count($priority); $i++)
		{
			if (($priority[$i] == 'group') && (!$found))
			{
				//
				// Get Group Quota, if we find one, we have our quota
				//
				$sub_sql = "SELECT u.group_id FROM " . USER_GROUP_TABLE . " u, " . GROUPS_TABLE . " g 
				WHERE (g.group_single_user = 0) AND (u.group_id = g.group_id) AND (u.user_id = " . $user_id . ")" .
						" AND (u.group_id <> 0)";
				
			
				$sql = "SELECT l.quota_limit FROM " . QUOTA_TABLE . " q, " . QUOTA_LIMITS_TABLE . " l
				WHERE (q.group_id IN (" . $sub_sql . "))  AND (q.quota_type = " . $quota_type . ") 
				AND (q.quota_limit_id = l.quota_limit_id) ORDER BY l.quota_limit DESC LIMIT 1";
				
				$result = $m2f_db->getOne($sql);
				if (!is_db_error($result, $sql, __LINE__, __FILE__))
				{
					if (isset($result))
					{
						$limit = $result;
						$found = TRUE;
					}
				}
			}	

			if (($priority[$i] == 'user') && (!$found))
			{
				//
				// Get User Quota, if the user is not in a group or the group has no quotas
				//
				$sql = "SELECT l.quota_limit FROM " . QUOTA_TABLE . " q, " . QUOTA_LIMITS_TABLE . " l
				WHERE (q.user_id = " . $user_id . ") AND (q.user_id	<> 0) AND (q.quota_type = " . $quota_type . ") 
				AND (q.quota_limit_id = l.quota_limit_id) LIMIT 1";	

				$result = $m2f_db->getOne($sql);
				if (!is_db_error($result, $sql, __LINE__, __FILE__))			
				{			
					
					if (isset($result))
					{
						$limit = $result;
						$found = TRUE;
					}
				}
			}
		}

		if 	(!$found)
		{
			// Set Default Quota Limit
			$quota_id =  $attach_config['default_upload_quota'];	

			if ($quota_id == 0)
			{
				$limit = $attach_config['attachment_quota'];
			}
			else
			{
				$sql = "SELECT quota_limit FROM " . QUOTA_LIMITS_TABLE . "
				WHERE quota_limit_id = " . $quota_id . " LIMIT 1";
				$result = $m2f_db->getOne($sql);
				if (!is_db_error($result, $sql, __LINE__, __FILE__))	
				{
					if (isset($result))
					{
						$limit = $result;
						$found = TRUE;
					}
				}
			
				if (!$found)
				{
					$limit = $attach_config['attachment_quota'];
				}
			}
		}
		// Never exceed the complete Attachment Upload Quota	

		if ($limit > $attach_config['attachment_quota'])
		{
			$limit = $attach_config['attachment_quota'];
		}
		return $limit;
	}

}  //  end of class	


?>