<?php
/**
  * Mailing list abstraction class
  *
  * @author Patricio Anguita
  * @license http://opensource.org/licenses/gpl-license.php GNU Public License
  * @version $Revision: 1.153 $
  * @internal $Id: m2f_mailinglist.php,v 1.153 2006/06/30 11:47:51 georgeocrawford Exp $
*/

include_once ($m2f_root_path . 'm2f_common.php');

// Abstract mailinglist class ---------------------------------
class m2f_mailinglist
{
	// Class factory function
	function factory($mailinglist_version)
	{
		switch ($mailinglist_version)
		{
			case 'internal':
				return new m2f_internal_mailinglist();
			break;

			default:
		    $m2f_error_msg = "Cannot create m2f_mailinglist for version: $mailinglist_version";
		    m2f_log(PEAR_LOG_CRIT , $m2f_error_msg , __LINE__, __FILE__);
		    die;
		}
	}
}

// Concrete maling list classes ------------------------------
class m2f_internal_mailinglist
{

	// Methods

	// Sends a new mail on a given mailing list, on behalf of the sender, and returns a boolean if it was possible
	function new_mail(&$params, &$mail_msg , &$mailinglist_id, $edited=FALSE, &$post_id, &$topic_id, &$user_id, $direction, $moderate = FALSE)
	{
		global $m2f_db, $board_config, $m2f_config;

		$recipients = array();

		// Calculate defualt for users with no preference set
		$default_user_prefs = $m2f_config['_M2F_DEFAULT_EMAIL_PREFS_'];
						
		if ($params['_M2F_EXTERNAL_LIST_SERVER_'] == 0)
		{
			// get list of the user_ids of all the banned users
			$sql = 'SELECT ban_userid FROM ' . BANLIST_TABLE;
			$db_result =& $m2f_db->getCol($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);
			$banned_user_list = implode(',', $db_result);
		
			// If the mailinglist is configured so that the whole board is subscribed, get all the board email addresses
			$sql = "SELECT subscribe_board FROM " . _M2F_MAILINGLIST_TABLE_ . "
								WHERE mailinglist_id = '$mailinglist_id'";
			$db_result = $m2f_db->getOne($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);
			
			if ($db_result == 1) // Whole board is subscribed - no more checks needed
			{
				$sql = "SELECT user_email, m2f_user_prefs FROM " . USERS_TABLE . "
								WHERE user_id > '0' 
								AND user_active > '0'";
				if ($banned_user_list) $sql .= " AND user_id NOT IN ($banned_user_list)";
				$db_result =& $m2f_db->getAll($sql);
				check_db_result($db_result, $sql, __LINE__, __FILE__);

				foreach ($db_result as $row)
				{
					if ($row['m2f_user_prefs'] == -1) $row['m2f_user_prefs'] = $default_user_prefs;					
					$recipients[$row['user_email']] = $row['m2f_user_prefs'];
				}
			}
			else // whole board isn't subscribed - do more checks
			{
				// Build the recipient list: Get the mails of that mailing list id
				$sql = "SELECT u.user_email, s.user_id, u.m2f_user_prefs
								FROM " . _M2F_SUBSCRIPTION_TABLE_ . " s
								LEFT JOIN " . USERS_TABLE . " u
								USING (user_id)
								WHERE s.mailinglist_id = '$mailinglist_id'
								AND u.user_active > 0";						
				if ($banned_user_list) $sql .= " AND u.user_id NOT IN ($banned_user_list)";
								
				$db_result =& $m2f_db->getAll($sql);
				check_db_result($db_result, $sql, __LINE__, __FILE__);
						
				foreach ($db_result as $row)
				{
					// if the 'user_email' row is empty, it means the user doesn't exist in the users table. Hence, remove the user from the m2f subscriptions table
					if (empty($row['user_email'])) {
						m2f_log(PEAR_LOG_WARNING , "Unknown user found in subscriptions table - deleting user $row[user_id] from mailing list $mailinglist_id" , __LINE__ , __FILE__ );
						$sql = "DELETE FROM " . _M2F_SUBSCRIPTION_TABLE_ . " WHERE user_id = '$row[user_id]'";
						$db_result = $m2f_db->query($sql);
						check_db_result($db_result, $sql, __LINE__, __FILE__);
					}
					else {
						if ($row['m2f_user_prefs'] == -1) $row['m2f_user_prefs'] = $default_user_prefs;
						$recipients[$row['user_email']] = $row['m2f_user_prefs'];
					}
				}
	
				// check for outdated groups
				$sql = "SELECT s.group_id FROM " . _M2F_GROUP_SUBSCRIPTION_TABLE_ . " s 
								LEFT JOIN " . USER_GROUP_TABLE . " ug
								ON s.group_id = ug.group_id 
								WHERE ug.group_id IS NULL";
				
				$db_result =& $m2f_db->getAll($sql);
				check_db_result($db_result, $sql, __LINE__, __FILE__);
	
				if (!empty($db_result))
				{
					foreach ($db_result as $row)
					{
						m2f_log(PEAR_LOG_WARNING , "Unknown group found in group subscriptions table - deleting group $row[group_id] from mailing list $mailinglist_id" , __LINE__ , __FILE__ );
						$sql = "DELETE FROM " . _M2F_GROUP_SUBSCRIPTION_TABLE_ . " WHERE group_id = '$row[group_id]'";
						$db_result = $m2f_db->query($sql);
						check_db_result($db_result, $sql, __LINE__, __FILE__);
					}
				}
				
				
				// If the mailinglist has groups subscribed to it, add their email addresses to the list
				$sql = "SELECT u.user_email, u.m2f_user_prefs
								FROM " . USERS_TABLE . " u, " . USER_GROUP_TABLE . " ug, " . _M2F_GROUP_SUBSCRIPTION_TABLE_ . " s
								WHERE s.mailinglist_id = $mailinglist_id  
								AND s.group_id = ug.group_id  
								AND u.user_id = ug.user_id  
								AND ug.user_pending = '0'  
								AND u.user_id > '0'
								AND u.user_active > '0'";
				if ($banned_user_list) $sql .= " AND u.user_id NOT IN ($banned_user_list)";
	
				$db_result =& $m2f_db->getAll($sql);
				check_db_result($db_result, $sql, __LINE__, __FILE__);
	
				if (!empty($db_result))
				{
					foreach ($db_result as $row)
					{
						if ($row['m2f_user_prefs'] == -1) $row['m2f_user_prefs'] = $default_user_prefs;					
						$recipients[$row['user_email']] = $row['m2f_user_prefs'];
					}
				}
			}

			// remove original poster from list if the option is set
			if ($params['_M2F_ADD_AUTHOR_AS_RECIPIENT_'] == 0 && $user_id > 0)
			{
				$sql = "SELECT user_email FROM " . USERS_TABLE . "
								WHERE user_id = $user_id";
				$db_result =& $m2f_db->getOne($sql);
				check_db_result($db_result, $sql, __LINE__, __FILE__);
				
				if (array_key_exists($db_result, $recipients))
				{
					unset($recipients[$db_result]);
				}
				
  			m2f_log(PEAR_LOG_INFO , "Removed author's address - '$db_result' - from recipients list as requested." , __LINE__ , __FILE__ );
			}
		}	// end of not an external list server	
		
		// Yahoo Groups fix
		if ($params['_M2F_NO_REPLYTO_RECP_'])
		{
			$remove_target = isset($mail_msg->headers['reply-to']) ? $mail_msg->headers['reply-to'] : '';
			// remove lowercase reply-to header because it won't be overwritten
			unset($mail_msg->headers['reply-to']);
			// only keep email address
			$remove_target = preg_replace('#^(?U).*((?-U)[a-z0-9._-]+@[a-z0-9.-]+).*$#i', '\1', $remove_target);
			if ($remove_target !== '')
			{
				m2f_log(PEAR_LOG_DEBUG , "Removing reply-to address \"$remove_target\" from subscriber list" , __LINE__ , __FILE__ );
				// remove reply-to address
				unset($recipients[$remove_target]);
			}
		}  // end yahoo groups fix
		
		// check if there are any addresses left in case the removed reply-to address was the only one subscribed
		if ((count($recipients) > 0) || $params['_M2F_EXTERNAL_LIST_SERVER_'] == 1)
		{  
			// Get ML configuration
			$ml_config = $this->get_config($mailinglist_id);
		
			//  Clean and setup the mail obj
			m2f_log(PEAR_LOG_INFO , "Cleaning up mail headers" , __LINE__ , __FILE__ );
			m2f_log(PEAR_LOG_DEBUG , "Dirty headers" , __LINE__ , __FILE__ ,$mail_msg->headers);
	
			$headers = $mail_msg->headers;
			unset ($mail_msg->headers);
  		
  		
			// Set up the references header. If we have a forum post, add Message-ID 
			// of first and last post to References, and Message-ID of first post to In-Reply-To. 
			// If we have an email, add contents of old References (or old In-Reply-To if unset) 
			// to new References. Add contents of old In-Reply-To to new In-Reply-To if set.
			// Message ID is the same if it exists, if not create a new one
			// Reference: http://www.faqs.org/rfcs/rfc2822.html
			
			// either use the incoming mail's message ID, or create a new one.
			$mail_msg->headers['Message-ID'] = empty($headers['message-id']) ? ('<' . time() . '.m2f.' . $post_id.'@'.$board_config['server_name'].'>') : $headers['message-id'];
				
			// store in DB so we link this Message ID to a particular post
			$sql = "INSERT INTO " . _M2F_MESSAGE_ID_TABLE_ . " (message_id, logtime, post_id) VALUES ('" . $mail_msg->headers['Message-ID'] . "', '" . time() . "', '$post_id')"; 
			$db_result = $m2f_db->query($sql); 
			check_db_result($db_result, $sql, __LINE__, __FILE__); 
			
			// try and store hash in hash table. this will fail if the incoming message was an email, but is essential if incoming message was a post!
			$id = $m2f_db->nextId(_M2F_MESSAGE_HASH_TABLE_, TRUE);
			$sql = "INSERT INTO " . _M2F_MESSAGE_HASH_TABLE_ . 
							" (hash, logtime, tag, id)".
							" VALUES ('" . $mail_msg->headers['Message-ID'] . "', '" . time() . "', 'SNT', '$id')";
			$db_result = $m2f_db->query($sql); // don't check for error here - there may well be one!
			
			$first_post_message_id = $this->first_post_message_id($topic_id, $post_id);
			$last_post_message_id = $this->last_post_message_id($topic_id);
			$orig_references = array();
			
			if (!empty($headers['references']))
			{
				preg_match_all('/\<.*?\>/', $headers['references'], $matches);
				$orig_references = $matches[0];
			}
			else if (!empty($headers['in-reply-to']))
			{
				preg_match_all('/\<.*?\>/', $headers['in-reply-to'], $matches);
				$orig_references = $matches[0];
			}

			$references = implode(' ', array_unique(array_merge($orig_references, (array) $first_post_message_id, (array) $last_post_message_id)));

			if (!empty($references))
			{
				$mail_msg->headers['References'] = $references;
				m2f_log(PEAR_LOG_INFO , "Mail references header: " . $references, __LINE__ , __FILE__ );
			}
			
			$in_reply_to = empty($headers['in-reply-to']) ? $first_post_message_id : $headers['in-reply-to'];
			
			if (!empty($in_reply_to))
			{
				$mail_msg->headers['In-Reply-To'] = $in_reply_to;
				m2f_log(PEAR_LOG_INFO , "Mail In-Reply-To header: " . $in_reply_to , __LINE__ , __FILE__ );
			}

			$mail_msg->headers['Subject'] = $headers['Subject'];
		
			$sql = "SELECT distributionlist_id
							FROM " . _M2F_DISTRIBUTIONLIST_TABLE_ . "
							WHERE mailinglist_id = '$mailinglist_id'";
			$db_result = $m2f_db->getOne($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);
		
			$sql = "SELECT config_name, config_value
							FROM " . _M2F_DISTRIBUTIONLIST_CONFIG_TABLE_ . "
							WHERE distributionlist_id = '$db_result'";
			$db_result =& $m2f_db->getAll($sql);
			check_db_result($db_result, $sql, __LINE__, __FILE__);
  			
			$m2f_reply_prefixes = array();
			foreach ($db_result as $row)
			{
				switch ($row['config_name'])
				{
					case '_M2F_FROM_FIELD_':
						$m2f_from_field = $row['config_value'];
					break;
					case '_M2F_FROM_OTHER_ADDRESS_':
						$m2f_from_other_address = $row['config_value'];
					break;
					case '_M2F_SUBJECT_PREFIX_':
						$m2f_subject_prefix = $row['config_value'];
					break;
					case '_M2F_REPLY_PREFIXES_':
						$m2f_reply_prefixes[] = $row['config_value'];
					break;
					case '_M2F_ADD_SUBJECT_TAG_EDITED_POST_':
						$m2f_add_edited_subject_tag = $row['config_value'];
					break;
					case '_M2F_ADD_BODY_TAG_EDITED_POST_':
						$m2f_add_edited_body_tag = $row['config_value'];
					break;
				}
			}
			
			if (empty($m2f_from_other_address)) $m2f_from_other_address = $ml_config['list_email'];
  		
			$headers['username'] = str_replace('.', ' ', $headers['username']);
  		switch($m2f_from_field)
  		{
  			case 'user_address':
		  		if (!empty($headers['user_email']) && !empty($headers['username']))
		  		{
		  		  // From email taken from forum post user info
		  		  $mail_msg->headers['From'] = '"'.$headers['username'] . '" <' . $headers['user_email'] . '>';
		  		  m2f_log(PEAR_LOG_INFO , "FROM header taken from forum post user info" , __LINE__ , __FILE__ );
		  		}
		  		else
		  		{
  				  // Anonymous - Use Mailing List Address
  				  $mail_msg->headers['From'] = $ml_config['name'] . ' <' . $ml_config['list_email'] . '>';
  				  m2f_log(PEAR_LOG_INFO , "Using mailing list address ($ml_config[list_email]) as FROM header" , __LINE__ , __FILE__ );
  				}
  			break;
  			
  			case 'ml_address':
  				// Use Mailing List Address
					$mail_msg->headers['From'] = $ml_config['name'] . ' <' . $ml_config['list_email'] . '>';
					m2f_log(PEAR_LOG_INFO , "Using mailing list address ({$ml_config[list_email]}) as FROM header" , __LINE__ , __FILE__ );
				break;

				case 'user_name_ml_address':
					// Use Mailing List Address with user name
					global $lang;
					$sender_name = $lang['Guest'];
					if (!empty($headers['username']))
					{
						$sender_name = $headers['username'];
					}

					$mail_msg->headers['From'] = $sender_name . ' <' . $ml_config['list_email'] . '>';
					m2f_log(PEAR_LOG_INFO , "Using user name and mailing list address ({$mail_msg->headers['From']}) as FROM header" , __LINE__ , __FILE__ );
  			break;
  			
				case 'user_name_other_address':
					// Use Mailing List Address with user name
					global $lang;
					$sender_name = $lang['Guest'];
					if (!empty($headers['username']))
					{
						$sender_name = $headers['username'];
					}

					$mail_msg->headers['From'] = $sender_name . ' <' . $m2f_from_other_address . '>';
					m2f_log(PEAR_LOG_INFO , "Using user name and other address ($m2f_from_other_address) as FROM header" , __LINE__ , __FILE__ );
  			break;
  			
  			case 'other_address':
  				// Alternative address specified in dist. list config
  				$mail_msg->headers['From'] = $m2f_from_other_address;
  				m2f_log(PEAR_LOG_INFO , "Using FROM header as specified in dist. list configuration ($m2f_from_other_address)" , __LINE__ , __FILE__ );
  			break;
  		}
  		
			// Prefix subject with configured value
			if ('' !== $m2f_subject_prefix)
			{
				$mail_msg->headers['Subject'] = str_replace($m2f_subject_prefix . ' ' , '' , $mail_msg->headers['Subject']);
				$mail_msg->headers['Subject'] = $m2f_subject_prefix . ' ' . $mail_msg->headers['Subject'];
			}
			
			if ($edited === TRUE)
			{
				$m2f_add_edited_subject_tag = trim($m2f_add_edited_subject_tag);
				if ($m2f_add_edited_subject_tag) $mail_msg->headers['Subject'] .= ' ' . $m2f_add_edited_subject_tag;
				
				$m2f_add_edited_body_tag = trim($m2f_add_edited_body_tag);
				if ($m2f_add_edited_body_tag)
				{
					$mail_msg->html_body = str_replace('<body>' , "<body>\n\n<p class=\"cattitle\">" . trim($m2f_add_edited_body_tag) . "</p><br>\n", $mail_msg->html_body);
					$mail_msg->body = '****' . trim($m2f_add_edited_body_tag) . '****' . "\n\n\n". $mail_msg->body;
				}
			}

			// Remove all but the first found reply prefix
			$found = false;
			foreach ($m2f_reply_prefixes as $prefix)
			{
				if (preg_match_all("/($prefix)/i", $mail_msg->headers['Subject'], $matches, PREG_PATTERN_ORDER))
				{
					$limit = $found ? count($matches[1]) : count($matches[1]) -1;
					$mail_msg->headers['Subject'] = preg_replace("/$prefix/i", "", $mail_msg->headers['Subject'], $limit);
					$found = true;
				}
			}

			// Collapse multiple whitespace in subject (may be left over from cleanup)
			$mail_msg->headers['Subject'] = preg_replace("/[ \t]{2,}/", " ", $mail_msg->headers['Subject']);

			$mail_msg->headers['Reply-To'] = $ml_config['list_email'];
			$mail_msg->headers['X-Generated-By'] = 'M2F: www.mail2forum.com';
			$mail_msg->headers['Date'] = date("r");

			// Replace bad words in the subject and body
			$orig_word = array();
			$replacement_word = array();
			obtain_word_list($orig_word, $replacement_word);
			if (count($orig_word)) {
				$mail_msg->headers['Subject'] = preg_replace($orig_word, $replacement_word, $mail_msg->headers['Subject']);
				$mail_msg->body = preg_replace($orig_word, $replacement_word, $mail_msg->body);
				$mail_msg->html_body = preg_replace($orig_word, $replacement_word, $mail_msg->html_body);
			}
		
			m2f_log(PEAR_LOG_DEBUG , "Cleaned up headers" , __LINE__ , __FILE__ ,$mail_msg->headers);
			m2f_log(PEAR_LOG_DEBUG , "Body" , __LINE__ , __FILE__ ,$mail_msg->body);


			$mail_msg->list_email = $ml_config['list_email'];

			if ($params['_M2F_EXTERNAL_LIST_SERVER_'] == 1)
			{
				// set the recipients to the list address
				$recipients = array ( $mail_msg->list_email => $default_user_prefs);
			}
			
			if (($direction == 'out') && ($moderate === TRUE))
			{
				// Store it
				return array($mail_msg, $recipients);
			}
			else
			{
				$mail_msg->send_by_pref($params, $recipients);
			}  		
		}
		
		return FALSE; // mail was not created
	}
	
	// get the message-id of the first post in a topic
	function first_post_message_id($topic_id, $post_id) 
	{ 
		global $m2f_db; 
		
		$sql = "SELECT MIN(post_id) FROM " . POSTS_TABLE . " WHERE topic_id = $topic_id"; 
		$db_result = $m2f_db->getOne($sql); 
		check_db_result($db_result, $sql, __LINE__, __FILE__); 
	
		// in this case, we have the first post of a new topic, so no References header today!
		if (empty($db_result) || $db_result == $post_id) return '';
	
		$sql = "SELECT message_id FROM " . _M2F_MESSAGE_ID_TABLE_ . " WHERE post_id = $db_result"; 
		$db_result = $m2f_db->getOne($sql); 
		check_db_result($db_result, $sql, __LINE__, __FILE__); 
		return $db_result; 
	}
	
	// get the message-id of the most-recent-but-1 post in a topic
	function last_post_message_id($topic_id) 
	{ 
		global $m2f_db; 

		$sql = "SELECT post_id FROM " . POSTS_TABLE . " WHERE topic_id = $topic_id ORDER BY post_id DESC"; 
		$db_result = $m2f_db->getCol($sql); 
		check_db_result($db_result, $sql, __LINE__, __FILE__); 

		if (empty($db_result[1])) return '';
		
		$sql = "SELECT message_id FROM " . _M2F_MESSAGE_ID_TABLE_ . " WHERE post_id = " . $db_result[1]; 
		$db_result = $m2f_db->getOne($sql); 
		check_db_result($db_result, $sql, __LINE__, __FILE__); 
		return $db_result;
	}


	// converts multiple quote levels into [quote] tags
	function convert_quotes_full($body)
	{

		$lines = explode("\n", $body);
		$quote_level = 0;
		foreach ($lines as $line)
		{
			// use this instead of strspn($line, '>') to allow for spaces
			preg_match('/^([> ]*)(.*)$/', $line, $matches);
			$num_quotes = substr_count($matches[1], '>');
			$line = $matches[2];
			
			if ($num_quotes > 0)
			{
				if ($num_quotes == $quote_level)
				{
					$new_lines[] = $line;
				}
				elseif ($num_quotes > $quote_level)
				{
					while ($num_quotes > $quote_level)
					{
						$line = '[quote]' . $line;
						$quote_level++;
					}
					$new_lines[] = $line;
				}
				elseif ($num_quotes < $quote_level)
				{
					while ($num_quotes < $quote_level)
					{
						$line = '[/quote]' . $line;
						$quote_level--;
					}
					$new_lines[] = $line;
				}
			}
			else
			{
				while ($quote_level > 0)
				{
					$line = '[/quote]' . $line;
					$quote_level--;
				}
				$new_lines[] = $line;
			}
		}
		$result = implode($new_lines, "\n");
		
		// strip any remaining empty quote sections - MUST be a recursive scan (says George!)
		$clean_result = preg_replace('/\[quote\]\s*(&nbsp;)?\s*\[\/quote\]/', '', $result); 
		while ($clean_result != $result)
		{
			$result = $clean_result;
			$clean_result = preg_replace('/\[quote\]\s*(&nbsp;)?\s*\[\/quote\]/', '', $result);
		}
		
		//make sure enough [quote] areas are closed peoperly - George encountered this problem when 
		//crappy emails have a <blockquote> tag but no corresponding </blockquote>
		$open = substr_count($result, '[quote]');
		$closed = substr_count($result, '[/quote]');
		if ($open > $closed) { $result .= str_repeat('[/quote]', ($open - $closed)); }
				
		return $result;
	}
	
	// maximum of one quote level is marked with [quote]. All further levels retain original > quoting
	function convert_quotes_partial($body)
	{
		$lines = explode("\n", $body);
		$quote_level = 0;
		foreach ($lines as $line)
		{
			if (substr($line, 0, 1) == '>')
			{
				$new_lines[] = ($quote_level == 0) ? '[quote]' . substr($line, 1) : substr($line, 1);
				$quote_level = 1;
			}
			else
			{
				$new_lines[] = ($quote_level == 1) ? $line . '[/quote]' : $line;
				$quote_level = 0;
			}
		}
		return (implode($new_lines, "\n"));
	}
	
	// Returns a message post object which will be posted on the forum, properly formatted
	function mail_to_post($mail_msg, $params, $forum_id, $distributionlist_id)
	{
    global $m2f_db , $m2f_config, $board_config, $m2f_handler;

		// Build forum msg
		$forum_msg = new forum_msg();



    // Use email subject as the title
    $subject = htmlspecialchars(trim($mail_msg->headers['subject']));
		
		if ($subject != '')
		{
			// Remove any specified subject prefix. This will take out any ocurrence of this tag on the whole subject.
			$subject = $this->remove_subject_prefix($subject, $params['_M2F_SUBJECT_PREFIX_']);

			// Remove subject prefixes, for example Re: and Fw:, also Yahoo Groups inserted group name "[GroupName]"
			// This will remove specified prefixes only from the start of the subject.
			$subject = $this->remove_reply_prefixes($subject, $params['_M2F_REPLY_PREFIXES_']);
		}

		// Replace blank subjects with config value, otherwise topic is unviewable in phpBB
		if ($subject == '')
		{
			$forum_msg->title = _M2F_BLANK_SUBJECT_TEXT_;
		}
		else
		{
			$forum_msg->title = $subject;
		}



		// Try to convert the HTML part of the message if it exists
		// Or transform plain text into HTML anyway if set like that
		if (!empty($mail_msg->html_body))
		{
		  m2f_log(PEAR_LOG_INFO , "Setting forum body as email HTML body", __LINE__ , __FILE__ );
			$forum_msg->body = $mail_msg->html_body;
			
			// Remove pattern matches
			m2f_log(PEAR_LOG_INFO , "Running Search & Replace patterns", __LINE__ , __FILE__ );
			$m2f_handler->run_patterns($forum_msg, $distributionlist_id);

			if ($board_config['allow_bbcode'])
			{
				m2f_log(PEAR_LOG_INFO , "Converting HTML email to bbcode", __LINE__ , __FILE__ );
				$forum_msg->body = $this->html2bb($forum_msg->body);
			}
			else
			{
				m2f_log(PEAR_LOG_INFO , "Converting HTML email to text", __LINE__ , __FILE__ );
				$forum_msg->body = $this->html2txt($forum_msg->body);
			}
		}
		else
		{
		  m2f_log(PEAR_LOG_INFO , "Setting forum body as msg body without convertion", __LINE__ , __FILE__ );
			$forum_msg->body = $mail_msg->body;

			// Remove pattern matches
			m2f_log(PEAR_LOG_INFO , "Running Search & Replace patterns", __LINE__ , __FILE__ );
			$m2f_handler->run_patterns($forum_msg, $distributionlist_id);
		}

		if ($board_config['allow_bbcode'])
		{
			// Convert quotes to bbquotes
			$forum_msg->body = $this->convert_quotes_full($forum_msg->body);
	  }
	  
		// Add a M2F signature?
		$m2f_post_sig = $m2f_config['_M2F_FORUM_POST_SIGNATURE_'];
		if ($m2f_post_sig != '')
		{
		  $forum_msg->body .= "\r\n\r\n " . $m2f_post_sig . " \r\n";
		}





		
		// Get date
		$date_found = FALSE;

		if ($params['_M2F_EXTRACT_MAIL_DATES_'] == 1)
		{
			if (count($mail_msg->headers['received']) > 0)
			{
				// Use date/time on first "Received" header. This is the timestamp from the last
				// mail server and should be more accurate than the client's "Date:" header.
				$rec_hdrs = (array) $mail_msg->headers['received'];
				foreach ($rec_hdrs as $rec_hdr)
				{
					$result = ($rec_pos = strpos($rec_hdr, ';')) ? strtotime(substr($rec_hdr, $rec_pos + 1)) : strtotime($rec_hdr);
					if ($result != -1)
					{
						$forum_msg->datetime = $result;
						m2f_log(PEAR_LOG_INFO , "Extracting date from the mail: " . $forum_msg->datetime , __LINE__ , __FILE__ );
						$date_found = TRUE;
						break;
					}
				}
			}
			
			if (!$date_found && trim($mail_msg->headers['date']) != '')
			{
				$rec_hdr = trim($mail_msg->headers['date']);
				$result = ($rec_pos = strpos($rec_hdr, ';')) ? strtotime(substr($rec_hdr, $rec_pos + 1)) : strtotime($rec_hdr);
				if ($result != -1)
				{
					$forum_msg->datetime = $result;
					m2f_log(PEAR_LOG_INFO , "Extracting date from the mail: " . $forum_msg->datetime , __LINE__ , __FILE__ );
					$date_found = TRUE;
				}
			}
		}
		
		if (!$date_found)
		{
			$forum_msg->datetime = time(); 
			m2f_log(PEAR_LOG_INFO , "Email date not found. Set to 'now': " . $forum_msg->datetime , __LINE__ , __FILE__ );
		}

		
		
		
		
		
		
		// try to guess topic_id from References: 
		if (isset($mail_msg->headers['references']) || isset($mail_msg->headers['in-reply-to'])) 
		{ 
			$to_test = $mail_msg->headers['references'] . $mail_msg->headers['in-reply-to'];
			
			// Extract id 
			preg_match_all('/\<.*?\>/', $to_test, $matches);
			
			$forum_msg->topic_id = $this->find_ref($matches[0], $forum_id); 
			
			if (!empty($forum_msg->topic_id))
			{
				m2f_log(PEAR_LOG_INFO , "Topic id detected through References: " . $forum_msg->topic_id , __LINE__ , __FILE__ ); 
				m2f_log(PEAR_LOG_DEBUG , "mail_msg->headers['references']" , __LINE__ , __FILE__ , $mail_msg->headers['references']); 
			}
		} 

    // If a topic hasn't been found yet, try to thread based on subject
    if (empty($forum_msg->topic_id))
		{
			m2f_log(PEAR_LOG_INFO , "Topic id not found yet (no references header). Attempting to thread by subject \"$subject\"" , __LINE__ , __FILE__ );
			
			// This will cause "No title" subjects to not thread together
			if ($subject != '')
			{
				// Subjects in the database are restricted in length, so chop off the search string if necessary, to the length of the phpBB topic_title column
				$field_length = 60; // phpBB default
				if ($m2f_db->dbsyntax == 'mysql')
				{
					$res  =& $m2f_db->query('SELECT topic_title FROM ' . TOPICS_TABLE);
					$info =  $m2f_db->tableInfo($res);
					$field_length = $info[0]['len'];
				}
				else if ($m2f_db->dbsyntax == 'pgsql')
				{
					$sql = "SELECT a.atttypmod AS len
									FROM pg_attribute a , pg_class c, pg_type t 
									WHERE c.relname = '" . TOPICS_TABLE . "'
									AND a.attrelid = c.oid AND a.atttypid = t.oid
									AND  a.attname = 'topic_title'";
					$field_length = $m2f_db->getOne($sql) - 4; // take off 4 bytes added to varchar fields
					check_db_result($field_length, $sql, __LINE__, __FILE__);
				}

				$forum_msg->title = substr($subject, 0, $field_length);
				$subject = substr(addslashes($forum_msg->title), 0, $field_length);
				
				// Set the amount of time to go back looking for matching topics
				// This should be configurable, but is hard-coded to 2 weeks right now (60 * 60 * 24 * 14)
				$time_limit = $forum_msg->datetime - 1209600;
				
				// Find the most recent topic with the same subject
				// Note: This uses topic_last_post_id, which is the last post made in this topic,
				//   not necessarily the latest time post. Still, this will work in 99.9% of cases.
				
				$sql = "SELECT t.topic_id
								FROM " . TOPICS_TABLE . " AS t,
								" . POSTS_TABLE . " AS p 
								WHERE t.topic_last_post_id = p.post_id
								AND t.topic_title LIKE '" . $subject . "'
								AND p.post_time > '$time_limit'
								AND t.forum_id = '$forum_id'
								ORDER BY p.post_time DESC LIMIT 1";
				
				// If nothing matches, result will be null, which will cause insert_post to create a new topic
				$db_result = $m2f_db->getOne($sql);
				check_db_result($db_result, $sql, __LINE__, __FILE__);
				
				$forum_msg->topic_id = $db_result;
				if ($forum_msg->topic_id)
				{
					m2f_log(PEAR_LOG_INFO , "Threading to topic ID $forum_msg->topic_id" , __LINE__ , __FILE__ );
				}
				else
				{
					m2f_log(PEAR_LOG_INFO , "Starting a new topic" , __LINE__ , __FILE__ );
				}
			}
		}
		return $forum_msg;
	}
	
	// find topic_id from references 
	function find_ref($refs, $desired_forum_id) 
	{ 
		global $m2f_db; 
		
		// loops the Reference IDs found in the References header.
		// for each, tries to find a corresponding post_id in  _M2F_MESSAGE_ID_TABLE_
		// for each match, first check that the post was in the desired forum
		// if it was, we've got our topic
		// if not, try all the other post_id matches
		
		foreach ($refs as $ref)
		{
			$sql = "SELECT post_id FROM " . _M2F_MESSAGE_ID_TABLE_ . " WHERE message_id = '".addslashes($ref)."'";  
			$post_ids = $m2f_db->getCol($sql); 
			check_db_result($post_id, $sql, __LINE__, __FILE__); 

			if (!empty($post_ids))
			{
				foreach ($post_ids as $post_id)
				{
					$sql = "SELECT topic_id, forum_id FROM " . POSTS_TABLE . " WHERE post_id = '$post_id'"; 
					$db_result = $m2f_db->getRow($sql); 
					check_db_result($db_result, $sql, __LINE__, __FILE__); 
					
					if ($db_result['forum_id'] == $desired_forum_id)
					{
						return $db_result['topic_id'];
					}
				}
			}
			return NULL;
		}
	} 
	
	// Remove any specified words from the subject
	function remove_subject_prefix($subject, $tag)
	{
		if (isset($tag))
		{
			$subject = trim(str_replace($tag, '', $subject));
		}
		return $subject;
	}

	// Remove specified prefixes from the subject, ignoring case	
	function remove_reply_prefixes($subject, $prefixes)
	{
		$pattern_prefixes = '';
		$separator = '';
		foreach ($prefixes as $prefix)
		{
			// Escape special preg chars from the prefix
			$prefix =preg_replace('/(\/|\\|\^|\$|\.|\[|\]|\||\(|\)|\?|\*|\+|\{|\})/','\\\${0}',$prefix);
			$pattern_prefixes .= $separator . $prefix;
			$separator = '|';
		}
		$pattern = '/^ *(('.$pattern_prefixes.') *)*/i';
    
    // Replace all prefixes that matches
    if ($pattern != '')
    {
      $subject = preg_replace($pattern,'',$subject);
	  }
	    
	  return $subject;
	}

  // Deletes a mailing list
	function delete_mailinglist ($mailinglist_id)
	{
	  global $m2f_db;
	  
		// Delete the mailing list
		$sql = "DELETE FROM " . _M2F_MAILINGLIST_TABLE_ . "
						WHERE mailinglist_id = '$mailinglist_id'";
		$db_result = $m2f_db->query($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
    
    // Delete the subscriptions
		$sql = "DELETE FROM " . _M2F_SUBSCRIPTION_TABLE_ . "
						WHERE mailinglist_id = '$mailinglist_id'";
		$db_result = $m2f_db->query($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);

    // Delete distributionlist
    $sql = "SELECT distributionlist_id FROM " . _M2F_DISTRIBUTIONLIST_TABLE_ . "
            WHERE mailinglist_id = " . $m2f_db->quoteSmart($mailinglist_id);
    $db_result = $m2f_db->getAll($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
    foreach ($db_result as $row)
    {
      m2f_handler::delete_distributionlist($row['distributionlist_id']);
    }
	}
	
	function get_mailinglist_name($mailinglist_id)
	{
	  global $m2f_db;
		$sql = "SELECT name FROM " . _M2F_MAILINGLIST_TABLE_ . "
						WHERE mailinglist_id = '$mailinglist_id'";
		$db_result = $m2f_db->getOne($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);

    return $db_result;
	}

	function get_mailinglist_address($mailinglist_id)
	{
	  global $m2f_db;
		$sql = "SELECT list_email FROM " . _M2F_MAILINGLIST_TABLE_ . "
						WHERE mailinglist_id = '$mailinglist_id'";
		$db_result = $m2f_db->getOne($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
    
    return $db_result;
	}
	
	// Gets subscription emails
  function get_subscriptions($mailinglist_id)
  {
    global $m2f_db;
  
    $sql = "SELECT u.username, u.user_id
    				FROM " . _M2F_SUBSCRIPTION_TABLE_ . " s,
    					" . USERS_TABLE . " u
    				WHERE u.user_id = s.user_id
    				AND s.mailinglist_id = '$mailinglist_id'
    				ORDER BY u.username";
    $db_result =& $m2f_db->getAll($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);

    return $db_result;
  }

	// Gets group subscriptions
  function get_group_subscriptions($mailinglist_id)
  {
    global $m2f_db;
  
    $sql = "SELECT u.username
    				FROM " . _M2F_GROUPS_SUBSCRIPTION_TABLE_ . " s,
    					" . USERS_TABLE . " u
    				WHERE u.user_id = s.user_id
    					AND s.mailinglist_id = '$mailinglist_id'";
    $db_result =& $m2f_db->getCol($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);

    return $db_result;
  }
  
  function get_userid_from_username ($username)
  {
  	global $m2f_db;
	  $sql = "SELECT user_id from " . USERS_TABLE . "
	  				WHERE username = '" . htmlspecialchars($username) . "'";
		$db_result = $m2f_db->getOne($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);

	  if ($db_result == null) return FALSE;

	  return $db_result;
  }
  
  // Subscribes a user to a mailinglist based on username
  function new_subscription_by_username ($mailinglist_id, $username)
  {
		$user_id = $this->get_userid_from_username($username);
		if (!$user_id) return FALSE;
		return $this->new_subscription($mailinglist_id, $user_id);
  }

  // Deletes a mailinglist subscription based on username
  function delete_subscription_by_username ($mailinglist_id, $username)
  {
		$user_id = $this->get_userid_from_username($username);
		if (!$user_id) return FALSE;
		return $this->delete_subscription($mailinglist_id, $user_id);
  }

	// Deletes a mailinglist subscription based on user id
	function delete_subscription ($mailinglist_id, $user_id)
	{
	  global $m2f_db;
	  
		// Delete the mailing list
		$sql = "DELETE FROM " . _M2F_SUBSCRIPTION_TABLE_ . "
						WHERE mailinglist_id = '$mailinglist_id'
							AND user_id = '$user_id'";
		$db_result = $m2f_db->query($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
    
    return $db_result;
	}
	
	// Subscribes a user to a mailinglist based on user id
	function new_subscription ($mailinglist_id, $user_id)
	{
	  global $m2f_db;
	  
	  // Check if the user is already subscribed
    $sql = "SELECT COUNT(*) FROM " . _M2F_SUBSCRIPTION_TABLE_ . "
    				WHERE mailinglist_id = '$mailinglist_id'
    					AND user_id = '$user_id'";
    $db_result = $m2f_db->getOne($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
    
    if ($db_result == 0)
    {
  		// Insert a new row
  		$sql = "INSERT INTO " . _M2F_SUBSCRIPTION_TABLE_ . "
  							(mailinglist_id, user_id)
  						VALUES ('$mailinglist_id', '$user_id')";
  		$db_result = $m2f_db->query($sql);
	    check_db_result($db_result, $sql, __LINE__, __FILE__);
	    return TRUE;
    }    
	}
	
	function count_subscriptions($mailinglist_id)
	{    
		return count($this->get_subscriptions($mailinglist_id));
	}

	function count_group_subscriptions($mailinglist_id)
	{
		global $m2f_db;
		$sql = "SELECT COUNT(*) FROM " . _M2F_GROUP_SUBSCRIPTION_TABLE_ . "
						WHERE mailinglist_id = '$mailinglist_id'";
		$db_result = $m2f_db->getOne($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
    
		return $db_result;
	}
	
	function delete_all_subscriptions($mailinglist_id)
	{
	  global $m2f_db;
	  $sql = "DELETE FROM " . _M2F_SUBSCRIPTION_TABLE_ . "
	  				WHERE mailinglist_id = '$mailinglist_id'";
    $db_result = $m2f_db->query($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
	}
		
	function set_whole_board_subscription($mailinglist_id)
	{
	  global $m2f_db;
	  
	  // Check if the user is already subscribed
    $sql = 'SELECT u.user_id FROM ' . USERS_TABLE . ' u 
    				LEFT JOIN ' . _M2F_SUBSCRIPTION_TABLE_ . ' s
    				ON u.user_id = s.user_id 
    				AND s.mailinglist_id = ' . $mailinglist_id . '
    				WHERE s.user_id IS NULL 
    				AND u.user_id != -1';

		$db_result = $m2f_db->getAll($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
    
		foreach ($db_result as $value)
		{
			$insert[] = "($mailinglist_id, $value[user_id])";
		}
		
		// If there are users to subscribe..
		if ($insert != "")
		{
  		m2f_log(PEAR_LOG_DEBUG , "Attempting to subscribe current whole board. insert " , __LINE__ , __FILE__ , $insert);
  		
  		$insert = implode(', ', $insert);
  
  		// Insert the new rows
  		$sql = "INSERT INTO " . _M2F_SUBSCRIPTION_TABLE_ . "
  							(mailinglist_id, user_id)
  						VALUES $insert";
  		$db_result = $m2f_db->query($sql);
  		check_db_result($db_result, $sql, __LINE__, __FILE__);
		}
	}

	function subscribe_group ($group_id, $mailinglist_id)
	{
	  global $m2f_db;
	  $sql = "INSERT INTO " . _M2F_GROUP_SUBSCRIPTION_TABLE_ . "
	  				(group_id, mailinglist_id) VALUES ($group_id, $mailinglist_id)";
    $db_result = $m2f_db->query($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
	}
	
	
	function unsubscribe_group ($group_id, $mailinglist_id)
	{
	  global $m2f_db;
	  $sql = "DELETE FROM " . _M2F_GROUP_SUBSCRIPTION_TABLE_ . "
	  				WHERE group_id = $group_id
	  				AND mailinglist_id = $mailinglist_id";
    $db_result = $m2f_db->query($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
	}
	
	function get_subscribed_groups($mailinglist_id)
	{
	  global $m2f_db;
		$sql = "SELECT g.group_id, s.mailinglist_id, g.group_name
			FROM " . GROUPS_TABLE . " g 
			LEFT JOIN " . _M2F_GROUP_SUBSCRIPTION_TABLE_ . " s
			ON (g.group_id = s.group_id
			AND s.mailinglist_id = $mailinglist_id)
			WHERE g.group_single_user <> 1
			ORDER BY g.group_name";

		$db_result = $m2f_db->getAll($sql);
		check_db_result($db_result, $sql, __LINE__, __FILE__);
		return($db_result);
	}

	function is_board_subscribed($mailinglist_id)
	{
	  global $m2f_db;
	  $sql = "SELECT subscribe_board FROM " . _M2F_MAILINGLIST_TABLE_ . "
	  				WHERE mailinglist_id = '$mailinglist_id'";
    $db_result = $m2f_db->getOne($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
    
		return $db_result;
	}
	
	function set_board_subscription($mailinglist_id, $subscribe)
	{
	  global $m2f_db;
	  $sql = "UPDATE " . _M2F_MAILINGLIST_TABLE_ . "
	  				SET subscribe_board = '$subscribe'
	  				WHERE mailinglist_id = '$mailinglist_id'";
    $db_result = $m2f_db->query($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
	}

	// Makes a new mailing lists, and returns a proper mailinglist_id
	// Params are equivalent to columns: name, etc
	function new_mailinglist ($params)
	{
	  global $m2f_db;
	  
		// Populate m2f_mailinglist, and get the new id
		$sql = "SELECT MAX(mailinglist_id) FROM " . _M2F_MAILINGLIST_TABLE_;
		$db_result = $m2f_db->getOne($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
    
		$new_id = $db_result + 1;
		
		// Insert a new row
		$sql = "INSERT INTO " . _M2F_MAILINGLIST_TABLE_ . "
							(mailinglist_id) VALUES ('$new_id')";
		$db_result = $m2f_db->query($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
		
		// Set new mailing list according to parameters
		m2f_log(PEAR_LOG_INFO , "Setting up new mailinglist " , __LINE__ , __FILE__ );
		m2f_log(PEAR_LOG_DEBUG , "params" , __LINE__ , __FILE__ ,$params);
		$this->set_mailinglist($new_id, $params);

		// Return the new id
		return ($new_id);

	}


	// Edits an existing mailing list
	function edit_mailinglist($mailinglist_id, $ml_name, $list_email)
	{
	  global $m2f_db;
	  
		// Insert a new row
		$sql = "UPDATE " . _M2F_MAILINGLIST_TABLE_ . "
						SET name = '$ml_name', list_email = '$list_email'
						WHERE mailinglist_id = '$mailinglist_id'";
		$db_result = $m2f_db->query($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
	}

	//Configures a mailing list, i.e. name, services, e-mail, etc (Obs: We should better define this)
	function set_mailinglist($mailinglist_id, $params)
	{
	  global $m2f_db;
	  
		// Foreach column in m2f_mailinglist table, update with $params
		// todo: Put default values if !isset($params['column_name'])
		
		// Insert a new row
		$sql = "UPDATE " . _M2F_MAILINGLIST_TABLE_ . " SET ";
		$comma = '';
		if (isset($params['name']))
		{
		  $sql .= "name = '{$params['name']}' ";
		  $comma = ',';
		}
		if (isset($params['list_email']))
		{
		  $sql .= $comma . "list_email = '{$params['list_email']}' ";
		  $comma = ',';
		}

		$sql .= " WHERE mailinglist_id = '$mailinglist_id' ";

		$db_result = $m2f_db->query($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);
	}
	
	// Retrieves mails from a mailinglist
	// Returns array of mail_msg objects
	function retrieve_msgs($params , $mailinglist_id)
	{
		require_once('Mail/mimeDecode.php');  // PEAR mimeDecode layer

		m2f_log(PEAR_LOG_INFO , "Retrieving messages from  mailinglist_id $mailinglist_id" , __LINE__ , __FILE__ );
		m2f_log(PEAR_LOG_INFO , "Using backend:" . $params['_M2F_INPUT_BACKEND_'], __LINE__ , __FILE__ );
				
		switch ($params['_M2F_INPUT_BACKEND_'])
		{
		  // Mails are stored in a mbox file
		  case 'mbox':
		    $mail_msgs = ($this->retrieve_mbox_msgs($params['_M2F_MBOX_FILE_'], $params['_M2F_MAX_MSG_RETRIEVE_']));
		    break;
		  // Mails are redirected to m2f via a forwarder
		  case 'stdin':
		    $mail_msgs = ($this->retrieve_stdin_msgs());
		    break;
		  // Mails must be retrieved from a pop account
		  case 'pop3':
		    $mail_msgs = ($this->retrieve_pop3_msgs($params['_M2F_POP3_HOST_'], $params['_M2F_POP3_PORT_'], $params['_M2F_POP3_USER_'], $params['_M2F_POP3_PASS_'], $params['_M2F_POP3_APOP_'], $params['_M2F_MAX_MSG_RETRIEVE_']));
		    break;  
		  // Mails must be retrieved from a gmail account
		  case 'gmail':
		    $mail_msgs = ($this->retrieve_gmail_msgs($params['_M2F_GMAIL_USERNAME_'], $params['_M2F_GMAIL_PASSWORD_'], $params['_M2F_MAX_MSG_RETRIEVE_']));
		    break;  
		}
		
		return $mail_msgs;	
	}
	

  // Retrieves mail from gMail account
	// Returns array of mail_msg objects
	function retrieve_gmail_msgs($username, $password, $max_msgs)
	{
		include_once ('includes/libgmailer.php');

    // Create array of mail_msgs
		$mail_msgs = array();

		$my_timezone = 0;
		$box = 'inbox';

		$gmail = new GMailer();
		if ($gmail->created) 
		{
			$gmail->setLoginInfo($username, $password, $my_timezone);
	    m2f_log(PEAR_LOG_INFO , "Logging in to gMail account", __LINE__ , __FILE__ );
			if ($gmail->connectNoCookie()) 
			{
				$q = "search=".$box."&view=tl&start=0";
				$gmail->fetch($q);
				$snapshot = $gmail->getSnapshot(GM_CONVERSATION); 
				
				if ($snapshot->created == 0) 
				{
					$m2f_error_msg = "Fail to get gMail inbox details. Last action: '" . $gmail->lastActionStatus() . "'";
					m2f_log(PEAR_LOG_CRIT , $m2f_error_msg , __LINE__ , __FILE__ );
					$this->gmail = $gmail; // this is so the connection can be closed later from outside this function
					return array();
				}  
				
				$processed = 0;
				
				m2f_log(PEAR_LOG_INFO , "Looping gMail conversations", __LINE__ , __FILE__ );
				foreach ($snapshot->box as $conversation)
				{
					$q = "search=".$box."&view=cv&th=".$conversation['id'];
					$gmail->fetch($q);
					$inner_snapshot = $gmail->getSnapshot(GM_CONVERSATION);  
					
					foreach ($inner_snapshot->conv as $email)
					{
						$id = $email['id'];
						$q = "view=om&th=" . $id;
						$mail_msgs[$id] = $gmail->dump($q);
						$processed++;

						if ($processed == $max_msgs) 
						{
							break 2;
						}
					}
				}
			} 
			else 
			{
				$m2f_error_msg = "Error connecting to gMail using '$username' '$password'. " . $gmail->lastActionStatus();
				m2f_log(PEAR_LOG_CRIT , $m2f_error_msg , __LINE__ , __FILE__ );
			}
		} 
		else 
		{
			$m2f_error_msg = "Failed to create GMailer class: " . $gmail->lastActionStatus();
			m2f_log(PEAR_LOG_CRIT , $m2f_error_msg , __LINE__ , __FILE__ );
		}
		
    m2f_log(PEAR_LOG_INFO , "Retrieved $processed gMail messages", __LINE__ , __FILE__ );
		$this->gmail = $gmail; // this is so the connection can be closed later from outside this function
		return $mail_msgs;
	}
	
  // Retrieves one mail from stdin, for use with a forwarder
	// Returns array of mail_msg objects
	function retrieve_stdin_msgs()
	{	  
	  // get message
	  m2f_log(PEAR_LOG_INFO , "Reading from STDIN for .forward incoming mail" , __LINE__ , __FILE__ );

		if (function_exists('file_get_contents'))
		{
	  	$thisMessage =  file_get_contents('php://stdin');
	  }
	  else
	  {
      $thisMessage = '';
      $i = fopen('php://stdin', 'r');
      
      // STDIN returns one packet at a time, so loop through all of the packets until EOF
      while (!feof($i)) {
        $thisMessage .= fread($i, 2000000);
      }
      fclose($i);
    }

    m2f_log(PEAR_LOG_DEBUG , "thisMessage" , __LINE__ , __FILE__ , $thisMessage );
    
		if ($thisMessage == '')
		{
			m2f_log(PEAR_LOG_INFO , "STDIN was empty" , __LINE__ , __FILE__ );
			return array();
		}

    return array($thisMessage);
	}



  // Retrieves mails from a pop3 account
	// Returns array of mail_msg objects
	function retrieve_pop3_msgs($host, $port, $user, $pass ,$apop, $max_msgs)
	{
		include_once ('Net/POP3.php');
		
	  // open POP connection
	  $pop3 =& new Net_POP3();
	  
	  // Uncomment this line to debug the POP connection:
	  //$pop3->setDebug();
	  
	  if (!$pop3->connect($host, $port))
	  {
	    $m2f_error_msg = "Error connecting to POP3 server $host:$port";
      m2f_log(PEAR_LOG_CRIT , $m2f_error_msg , __LINE__ , __FILE__ );
	  }
	  
	  $result = $pop3->login($user, $pass , ($apop == '1' ? 'APOP' : 'USER'));
	  if (PEAR::isError($result))
	  {
	    $m2f_error_msg = 'Error logging into POP3 server: ' . $result->getMessage();
      m2f_log(PEAR_LOG_CRIT , $m2f_error_msg , __LINE__ , __FILE__ );
	  }
	  
	  // get total messages
    $total = $pop3->numMsg();
    if ($total === false)
    {
	    $m2f_error_msg = 'Error retrieving number of POP messages';
      m2f_log(PEAR_LOG_CRIT , $m2f_error_msg , __LINE__ , __FILE__ );
    }
    
    $total = ($max_msgs == 0) ? $total : (($max_msgs < $total) ? $max_msgs : $total); // break only if $max_msgs isn't 0 (which is unlimited import number)
    
    // Create array of mail_msg objects
		$mail_msgs = array();

    m2f_log(PEAR_LOG_INFO , "Retrieving $total POP messages", __LINE__ , __FILE__ );
		
		// For each mail in the mailbox (up to the limit)
		for ($x = 1; $x <= $total; $x++)
		{
			$thisMessage = $pop3->getMsg($x);
			if ($thisMessage === false)
			{
		    $m2f_error_msg = 'Error retrieving message number ' . $x;
	      m2f_log(PEAR_LOG_CRIT , $m2f_error_msg , __LINE__ , __FILE__ );
			}
			$mail_msgs[$x] = $thisMessage;
		}
		
		$this->pop3 = $pop3; // this is so the connection can be closed later from outside this function
    return $mail_msgs;
	}
	
	
	
	// Retrieves mails from a mbox file
	// Returns array of mail_msg objects
	function retrieve_mbox_msgs($mbox_file, $max_msgs)
	{
		include_once ('includes/mbox_m2f.php');
		$this->mbox =& new Mail_Mbox_m2f();

		// If the lock file exists, exit for now
		if (file_exists($mbox_file . '.lock'))
		{
			m2f_log(PEAR_LOG_CRIT , 'Lock file exists at \'' . $mbox_file . '.lock\'. Try again later. You may delete this file if you are sure no other script is trying to read the mBox.', __LINE__ , __FILE__ );
			return array();
		}

		// Create lock file for mbox
		// This is how procmail single threads writing to the mbox file. Not sure if this is universal.
		if (!(@touch($mbox_file . '.lock')))
		{
			m2f_log(PEAR_LOG_CRIT , "Failed creating lock file", __LINE__ , __FILE__ );
			return array();
		}

		$this->mbox_id = @$this->mbox->open($mbox_file, $max_msgs);
		
		if (PEAR::isError($this->mbox_id)) 
		{
			m2f_log(PEAR_LOG_CRIT , "Failed opening mbox file. " . $this->mbox_id->getMessage(), __LINE__ , __FILE__ );
			unlink($mbox_file . '.lock');
			return array();
		}
			
		$mail_msgs = $this->mbox->retrieve($this->mbox_id);
		$num_msgs = count($mail_msgs);
		
		if ($num_msgs == 0)
		{
			// Empty mbox
			m2f_log(PEAR_LOG_INFO , "Mbox file empty, or not in Mbox format. No messages processed", __LINE__ , __FILE__ );
			return array();
		}
		
    m2f_log(PEAR_LOG_INFO , "Retrieved $num_msgs Mbox messages", __LINE__ , __FILE__ );
		
		return $mail_msgs;
	}
	


/*
	// Converts the raw mail message to a M2F mail message 
	function add_mail_body($raw_msg, &$mail_msg) 
	{ 
		// Parameters for  mail decoding 
		$params = array(); 
		$params['include_bodies'] = TRUE; 
		$params['decode_bodies']  = TRUE; 
		$params['decode_headers'] = FALSE; 
		$params['input'] = $raw_msg; 
			
		m2f_log(PEAR_LOG_INFO , "MIMEDecoding mail body" , __LINE__ , __FILE__ ); 
		$structure = Mail_mimeDecode::decode($params); 
		
		m2f_log(PEAR_LOG_DEBUG , "body: ", __LINE__ , __FILE__ , $structure); 
		
		if (strtolower($structure->ctype_primary) == 'text') 
		{ 
			m2f_log(PEAR_LOG_INFO , "Mail has a text body" , __LINE__ , __FILE__ ); 
			if (strtolower($structure->ctype_secondary) == 'plain') 
			{          
				$mail_msg->body = $structure->body; 
			} 
			elseif (strtolower($structure->ctype_secondary) == 'html') 
			{ 
				$mail_msg->html_body = $structure->body; 
			} 
			else 
			{ 
				m2f_log(PEAR_LOG_INFO , "Structure of email was not recognized: " . 
				$structure->ctype_primary . ", " . $structure->ctype_secondary , __LINE__ , __FILE__ ); 
			} 
		} 
		else if (strtolower($structure->ctype_primary) == 'multipart') 
		{ 
			$mail_msg->body = ''; 
			$mail_msg->html_body = ''; 
			$html_found = false; 
			$text_only = false; 
			$this->decode_multipart( $structure->parts, $mail_msg, $html_found, $text_only); 
			m2f_log(PEAR_LOG_DEBUG , "mail_msg->body: " , __LINE__ , __FILE__ , $mail_msg->body ); 
			m2f_log(PEAR_LOG_DEBUG , "mail_msg->html_body: " , __LINE__ , __FILE__ , $mail_msg->html_body );              
		} 
		else
		{
			m2f_log(PEAR_LOG_INFO , "Mail has a ctype error - Defaulting to text" , __LINE__ , __FILE__ , $structure->body ); 
			$mail_msg->body = $structure->body; 
		} 
	} 

	function convert_mail_headers($raw_msg) 
	{ 
		// Parameters for  mail decoding 
		$params = array(); 
		$params['include_bodies'] = FALSE; 
		$params['decode_bodies']  = FALSE; 
		$params['decode_headers'] = TRUE; 
		$params['input'] = $raw_msg; 
			
		m2f_log(PEAR_LOG_INFO , "MIMEDecoding mail" , __LINE__ , __FILE__ ); 
		$structure = Mail_mimeDecode::decode($params); 
		m2f_log(PEAR_LOG_DEBUG , "headers: ", __LINE__ , __FILE__ , $structure->headers); 
		
		return $structure->headers; 
	} 
*/	



	// Converts the raw mail message to a M2F mail message 
	function decode_raw_message($raw_msg, &$mail_msg) 
	{ 
		// Parameters for  mail decoding 
		$params = array(); 
		$params['include_bodies'] = TRUE; 
		$params['decode_bodies']  = TRUE; 
		$params['decode_headers'] = TRUE; 
		$params['input'] = $raw_msg; 
			
		m2f_log(PEAR_LOG_INFO , "MIMEDecoding raw mail" , __LINE__ , __FILE__ ); 
		$structure = Mail_mimeDecode::decode($params); 
		
		m2f_log(PEAR_LOG_DEBUG , "decoded message: ", __LINE__ , __FILE__ , $structure); 
		
		$mail_msg->headers = $structure->headers;
		
		if (strtolower($structure->ctype_primary) == 'text') 
		{ 
			m2f_log(PEAR_LOG_INFO , "Mail has a text body" , __LINE__ , __FILE__ ); 
			if (strtolower($structure->ctype_secondary) == 'plain') 
			{          
				$mail_msg->body = $structure->body; 
			} 
			elseif (strtolower($structure->ctype_secondary) == 'html') 
			{ 
				$mail_msg->html_body = $structure->body; 
			} 
			else 
			{ 
				m2f_log(PEAR_LOG_INFO , "Structure of email was not recognized: " . 
				$structure->ctype_primary . ", " . $structure->ctype_secondary , __LINE__ , __FILE__ ); 
			} 
		} 
		else if (strtolower($structure->ctype_primary) == 'multipart') 
		{ 
			$mail_msg->body = ''; 
			$mail_msg->html_body = ''; 
			$html_found = false; 
			$text_only = false; 
			$this->decode_multipart( $structure->parts, $mail_msg, $html_found, $text_only); 
			m2f_log(PEAR_LOG_DEBUG , "mail_msg->body: " , __LINE__ , __FILE__ , $mail_msg->body ); 
			m2f_log(PEAR_LOG_DEBUG , "mail_msg->html_body: " , __LINE__ , __FILE__ , $mail_msg->html_body );              
		} 
		else
		{
			m2f_log(PEAR_LOG_INFO , "Mail has a ctype error - Defaulting to text" , __LINE__ , __FILE__ , $structure->body ); 
			$mail_msg->body = $structure->body; 
		} 
	} 



	
	function is_multipart(&$part) 
	{ 
		return  (strtolower($part->ctype_primary) == 'multipart' ); 
	} 

	function is_text(&$part) 
	{ 
		return  (strtolower($part->ctype_primary) == 'text' ); 
	} 

	function is_message(&$part) 
	{ 
		if ((strtolower($part->ctype_primary) == 'message') && (strtolower($part->ctype_secondary) == 'rfc822')) 
		{ 
			return true; 
		} 
		// TO DO return true for attachments with .eml extension 
		return false; 
	} 

	function is_attachment(&$part) 
	{ 
		return (isset($part->disposition) and (strtolower($part->disposition)=='attachment') 
			or ((strtolower($part->disposition)=='inline') and (strtolower($part->ctype_primary) != 'text')) 
			or (strtolower($part->ctype_primary) == 'image')); 
	} 
                          
	// merge the $new html into the body of $existing 
	function html_merge(&$existing, $new) 
	{ 
		// is there anything there already? 
		if ($existing == '')
		{ 
			return $new; 
		} 
		else 
		{ 
			// find the relevant part of $new 
			$new_body = array(); 
			if ( preg_match('#<body[^>]*?>(.*?)</body>#is', $new, $new_body) ) // leave this here for George please! --- <?   
			{ 
				// found the body part 
				$text_to_add = $new_body[1]; 
			} 
			else 
			{ 
				// no body part so use the whole thing 
				$text_to_add = $new; 
			} 
			
			// find out where to place it and place it 
			if (stristr($existing, '</body>') === FALSE) 
			{ 
				// no sensible place to put it so concatenate 
				return $existing . $text_to_add; 
			} 
			else 
			{ 
				// just before the </body> 
				return preg_replace( '#</body>#i', $text_to_add . '</body>', $existing); 
				//   return str_ireplace( '</body>', $text_to_add . '</body>', $existing ); 
			}    
		} 
	} 
	
       
      
	function decode_message(&$msg, &$original, &$txt_only_found) 
	{ 
		$html_found = false; // did we have html in this message (not including attached messages) 
		
		// set up header 
		if (isset($msg->parts[0]->headers['subject'])) 
		{ 
			$header = '------ Start of attached email. Subject: ' . $msg->parts[0]->headers['subject'] . ' ------'; 
		} 
		else 
		{ 
			$header = '------ Start of attached email ------'; 
		} 
		
		$original->body .= "\r\n\r\n" . $header . "\r\n"; 
		
		//cannot make sense of html within html so dump it all 
		$original->html_body = ''; 
		$txt_only_found = true; 
		
		// decode the message            
		$this->decode_multipart( $msg->parts, $original, $html_found, $txt_only_found ); 
		
		$trailer = '------ End of attached email ------'; 
		
		//plain 
		$txt = "\r\n" . $trailer . "\r\n"; 
		$original->body .= $txt;  
	}    
    
	function decode_multipart(&$subparts, &$msg, &$html_found, &$txt_only_found) 
	{ 
		$html_found = false; 
		
		if (empty($subparts))
		{ 
			$msg->html_body = ''; 
			return;
		} 

		foreach ( array_keys($subparts) as $key ) 
		{ 
			$part =& $subparts[$key]; 
			
			// is it a multipart? 
			if ($this->is_multipart($part))
			{ 
				$this->decode_multipart( $part->parts, $msg, $html_found, $txt_only_found); // recurse through the subparts 
			} 
			else if (($this->is_text($part)) and !($this->is_attachment($part)))// is it text ?   
			{ 
				switch (strtolower($part->ctype_secondary)) { 
					case 'plain': 
						$msg->body .= $part->body; 
						break; 
					case 'html': 
						$html_found = true;    
						$msg->html_body = $this->html_merge($msg->html_body, $part->body); 
						break; 
					default: 
						if (_M2F_ENABLE_ATTACHMENTS_ == 1)
						{ 
							m2f_log(PEAR_LOG_INFO , "Adding unrecognized text part " . $part->ctype_secondary  . " as attachment", __LINE__ , __FILE__ ); 
							$msg->attachments->add($part); 
						} 
						break;
				}
			} 
			else if ($this->is_message($part)) // embedded atatched message 
			{
				$this->decode_message($part, $msg, $txt_only_found); 
			} 
			else if ($this->is_attachment($part)) // attachment 
			{ 
				$trimmed = $part->body;
				
				// Weird catch for attachments which have been removed (Yahoo Groups only??), but are still marked as being base64 encoded
				if ( ($trimmed == '[Attachment content not displayed.]') || ($part->headers['content-transfer-encoding'] == 'base64' && $trimmed == base64_decode('[Attachment content not displayed.]')) )
				{
					m2f_log(PEAR_LOG_INFO , "Attachment was empty - marked as '[Attachment content not displayed.]' in the email." , __LINE__ , __FILE__ );
				}
				else if (_M2F_ENABLE_ATTACHMENTS_ == 1)
				{ 
					m2f_log(PEAR_LOG_INFO , "Adding attachment " . $part->ctype_secondary , __LINE__ , __FILE__ );
					$msg->attachments->add($part); 
				} 
			} 
		
		} // foreach 
		
		if (($txt_only_found) or (!$html_found)) // if we had a mix of html and plain emails, drop the html (if any) 
		{ 
			$msg->html_body = ''; 
		} 
	}
   


	// Recursive function to get the specified subpart of the MIME message.
	function get_subpart($subparts, $type, $subtype)
	{
		m2f_log(PEAR_LOG_INFO , "Trying to find $type/$subtype..." , __LINE__ , __FILE__ );
		foreach ($subparts as $part)
		{
			if (strtolower($part->ctype_primary) == $type &&
				strtolower($part->ctype_secondary) == $subtype)
			{
				return $part->body;
			}
			else if (strtolower($part->ctype_primary) == 'multipart')
			{
				$body = $this->get_subpart($part->parts, $type, $subtype);
				if (!is_null($body))
				{
					return $body;
				}
			}
		}
		m2f_log(PEAR_LOG_INFO , "$type/$subtype not found" , __LINE__ , __FILE__ );
	}
	
  // Get configuration directives for a mailing list id
	// Returns relational array
	function get_config($mailinglist_id)
	{
		global $m2f_db;

		// Read configuration from the DB
		$sql = "SELECT * FROM " . _M2F_MAILINGLIST_TABLE_ . "
						WHERE mailinglist_id = '$mailinglist_id'";
		$db_result =& $m2f_db->getRow($sql);
    check_db_result($db_result, $sql, __LINE__, __FILE__);

		return $db_result;
	}
	
	// HTML conversion common to both bbcode and text result
	function html_common($html2bbtxt)
	{
		// First extract just the body of the message
		if (stristr($html2bbtxt, "<body"))
		{
      if (preg_match('#<body.*?>(.*)</body>#is', $html2bbtxt, $matches))
      {
        $html2bbtxt = $matches[1];
      }
      elseif (preg_match('#<body.*?>(.*)#is', $html2bbtxt, $matches))
      {
        $html2bbtxt = $matches[1];
      }
		}
		
		// Convert all HTML tags to lower case
		$html2bbtxt = preg_replace('#(</?)(\w+)([^>]*>)#e',
																'"$1".strtolower("$2")."$3"',
																$html2bbtxt);
             
		//
		// Save the pre-formatted text
		//
		preg_match_all('#<pre(| .*?)>(.*?)</pre>#s', $html2bbtxt, $matches);
		$preformated_strings = $matches[2];
		$cp = count($preformated_strings);
		
		for ($i = 0; $i < $cp; $i++)
		{
			$html2bbtxt = preg_replace('#<pre(| .*?)>(.*?)</pre>#s', "***pre_string***$i", $html2bbtxt, 1);
		}
		
		//
		// Replace formatting elements
		//
		// Replace all CR LF with a single space
		$html2bbtxt = str_replace("\r\n", ' ', $html2bbtxt);
		// Then replace CR or LF alone with a single space
		$html2bbtxt = str_replace("\r", ' ', $html2bbtxt);
		$html2bbtxt = str_replace("\n", ' ', $html2bbtxt);
		// Replace line break with a CR LF
		$html2bbtxt = str_replace('<br>', "\r\n", $html2bbtxt);
		// The end of a division should trigger a line break
		$html2bbtxt = str_replace('</div>', "\r\n", $html2bbtxt);
		
		// replace spaces
		$html2bbtxt = str_replace('&nbsp;', ' ', $html2bbtxt);
		
		// replace multiple spaces
		$html2bbtxt = preg_replace('/ {2,}/', ' ', $html2bbtxt);
		
		
		// Outlook sometimes puts extra stuff with the paragraph marker, so match
		// <p> and <p otherjunk>
		$html2bbtxt = preg_replace('#<p(| .*?)>#', "\r\n", $html2bbtxt);

		//
		// Restore the preformatted text		
		//
		for ($i = 0; $i < $cp; $i++)
		{
			$html2bbtxt = str_replace("***pre_string***$i", '<pre>' . $preformated_strings[$i] . '</pre>', $html2bbtxt);
		}

		return $html2bbtxt;
	}
	
	// Converts an HTML email into bbcode		
	// This function is loosely based on cbparser.php by corz.org
	function html2bb($html2bbtxt)
	{	

		// Do common conversion stuff
		$html2bbtxt = $this->html_common($html2bbtxt);

		// Do simple string replacements
		$html2bbtxt = str_replace('</b>',  '[/b]',    $html2bbtxt);
		$html2bbtxt = str_replace('</i>',  '[/i]',    $html2bbtxt);
		$html2bbtxt = str_replace('</u>',  '[/u]',    $html2bbtxt);
		$html2bbtxt = str_replace('</ul>', '[/list]', $html2bbtxt);
		$html2bbtxt = str_replace('</ol>', '[/list]', $html2bbtxt);
		$html2bbtxt = str_replace('</em>', '[/i]',    $html2bbtxt);
		$html2bbtxt = str_replace('</strong>', '[/b]', $html2bbtxt);
		$html2bbtxt = str_replace('</blockquote>', '[/quote]', $html2bbtxt);
		$html2bbtxt = str_replace('</pre>', '[/quote]', $html2bbtxt);

		// Do simple reg expr replacements
		$html2bbtxt = preg_replace('#<b(| .*?)>#',      '[b]',      $html2bbtxt);
		$html2bbtxt = preg_replace('#<i(| .*?)>#',      '[i]',      $html2bbtxt);
		$html2bbtxt = preg_replace('#<u(| .*?)>#',      '[u]',      $html2bbtxt);
		$html2bbtxt = preg_replace('#<ul(| .*?)>#',     '[list]',   $html2bbtxt);
		$html2bbtxt = preg_replace('#<ol(| .*?)>#',     '[list=1]', $html2bbtxt);
		$html2bbtxt = preg_replace('#<li(| .*?)>#',     '[*]',      $html2bbtxt);
		$html2bbtxt = preg_replace('#<em(| .*?)>#',     '[i]',      $html2bbtxt);
		$html2bbtxt = preg_replace('#<strong(| .*?)>#', '[b]',      $html2bbtxt);
		$html2bbtxt = preg_replace('#<blockquote(| .*?)>#', '[quote]',  $html2bbtxt);
		$html2bbtxt = preg_replace('#<pre(| .*?)>#', '[quote]',  $html2bbtxt);

		// replace multiple instances of [b] or [i] with single tags
		$html2bbtxt = preg_replace('#(\[b\])+#',      '[b]',      $html2bbtxt);
		$html2bbtxt = preg_replace('#(\[i\])+#',      '[i]',      $html2bbtxt);
		$html2bbtxt = preg_replace('#(\[/b\])+#',     '[/b]',      $html2bbtxt);
		$html2bbtxt = preg_replace('#(\[/i\])+#',     '[/i]',      $html2bbtxt);

		// fix for thunderbird which chops up quotes into little chunks for some reason. Remove if necessary!
		$html2bbtxt = preg_replace('#\[\/quote\]\s*?\[quote\]#', '',  $html2bbtxt);

		// Replace email address
		$html2bbtxt = preg_replace('#<a .*?href=.*?"mailto:(.*?)".*?>(.*?)</a>#i', "$2 ([email]$1[/email])", $html2bbtxt);

		// Replace links
		$html2bbtxt = preg_replace('#<a .*href=.*"(.*)".*>(.*)</a>#iUe', "'[url'. (trim('$1') ? '='.trim('$1') : '') .']'.trim('$2').'[/url]'", $html2bbtxt);

		// Remove any image tags whose source starts with 'cid:' - this is an inline attachment, and will be added to the post as a normal attachment.
		$html2bbtxt = preg_replace('#<img[^>]+src="cid:[^>]+>#i', '', $html2bbtxt);

		// Replace image references
		$html2bbtxt = preg_replace('#<img .*src="(.*)".*>#iUe', "'[img]'.trim('$1').'[/img]'", $html2bbtxt);

		// Remove all remaining HTML tags
		$html2bbtxt = preg_replace('#<(/?\w+|!--)[^>]*>#', '', $html2bbtxt);

		// Convert HTML entities
		$html2bbtxt = html_entity_decode($html2bbtxt);

		// Convert quotes
		if (get_magic_quotes_gpc()) {
			return stripslashes($html2bbtxt);
		} else {
			return ($html2bbtxt);
		}
	}

	//
	// Function for converting an HTML email message into plain text
	//	
	function html2txt($html2bbtxt)
	{	
		// Do common conversion stuff
		$html2bbtxt = $this->html_common($html2bbtxt);
		
		// Replace links
		$html2bbtxt = preg_replace('#<a .*?href=.*?"(.*?)".*?>(.*?)</a>#', "$2 ($1)", $html2bbtxt);

		// Replace image references
		$html2bbtxt = preg_replace('#<img .*?src="(.*?)".*?>#', "(image: $1)", $html2bbtxt);
		
		// Convert blockquote
		$html2bbtxt = preg_replace('#<blockquote>#', "\r\nQuote:\r\n\r\n", $html2bbtxt);
		$html2bbtxt = preg_replace('#</blockquote>#', "\r\n(end of quote)\r\n", $html2bbtxt);
		
		// Remove all remaining HTML tags
		$html2bbtxt = preg_replace('#</?\w+[^>]*>#', '', $html2bbtxt);

		// Convert HTML entities
		$html2bbtxt = html_entity_decode($html2bbtxt);
		
		// Convert other common HTML entities with a close text value
		$match = array('&#8216;', '&#8217;', '&#8220;', '&#8221;', '&#8364;');
		$repl = array("'", "'", '"', '"', '');
		$html2bbtxt = str_replace($match, $repl, $html2bbtxt);
		
		// Convert quotes
		if (get_magic_quotes_gpc()) {
			return stripslashes($html2bbtxt);
		} else {
			return ($html2bbtxt);
		}
	}
}

?>